# === Treats all the configuration of the app ===

"""
All the configuration is loaded from this file

Currently we support the following 2 public functions and 4 private functions:

Public:

1. **getConfig** - Returns the configuration of some key values you explicitly tell on params (jump to section in [[Config.py#getConfig]] )

2. **getApiKey** - Return an API Key registered into configuration (jump to section in [[Config.py#getApiKey]] )

Private:

1. **__readConfig__** - Read's all the configuration included in your config.json file (jump to section in [[Config.py#__readConfig__]] )

2. **__save_config__** - Save configuration passed as parameter to this function. This, writes into your config.json (jump to section in [[Config.py#__save_config__]] )

3. **__createModuleTemplateConfig__** - This function creates into config.json a template with a simple config for trying in your GUI your functions (jump to section in [[Config.py#__createModuleTemplateConfig__]] )

4. **__look_for_changes__** - Reloads the configuration loaded (jump to section in [[Config.py#__look_for_changes__]] )

"""

import os as __os
import json as __json

global __config__
__config__ = {}

global __api_keys_sessions__
__api_keys_sessions__ = False

global __switching_to_map__
__switching_to_map__ = False

# === getConfig ===
def getConfig(parentKey, key, subkey=None, extrasubkey=None):
    """
    Returns the configuration of some key values
    you explicitly tell on params
    
    Arguments
    ---------
        parentKey : str
            
            Parent key of the config.json
        key : str
            
            The String of a child into that parent 
            key on the config.json
    
    Keyword Arguments
    -----------------
        subkey : str
            
            The String of a child, into that child, 
            into that parent key on the config.json 
            (default: {None})
        extrasubkey : str
            
            The String of a child, into that child, 
            into the other child, into that parent 
            key on the config.json (default: {None})
    
    Returns
    -------
        List
            
            The content of the config.json you selected 
            into an List / None
    """
    try:
        if extrasubkey:
            try:
                if subkey:
                    try:
                        return __config__[parentKey][key][subkey][extrasubkey]
                    except:
                        return
            except:
                return
        else:
            try:
                if subkey:
                    try:
                        return __config__[parentKey][key][subkey]
                    except:
                        return
            except:
                return
        return __config__[parentKey][key]
    except:
        return None

def add_pool_node(node):
    __config__ = {}
    with open(__os.path.join(__os.path.dirname(__file__) , 'config.json')) as json_data_file:
        __config__ = __json.load(json_data_file)

    if not 'known_nodes' in __config__['core']['Pool']:
        __config__['core']['Pool']['known_nodes'] = []

    __config__['core']['Pool']['known_nodes'].append(node)

    __save_config__(__config__)

def remove_pool_node(node):
    __config__ = {}
    with open(__os.path.join(__os.path.dirname(__file__) , 'config.json')) as json_data_file:
        __config__ = __json.load(json_data_file)

    if not 'known_nodes' in __config__['core']['Pool']:
        __config__['core']['Pool']['known_nodes'] = []
    
    if node in __config__['core']['Pool']['known_nodes']:
        __config__['core']['Pool']['known_nodes'].remove(node)

    __save_config__(__config__)

def add_my_service(node): # Used by Heroku
    __config__ = {}
    with open(__os.path.join(__os.path.dirname(__file__) , 'config.json')) as json_data_file:
        __config__ = __json.load(json_data_file)

    if not 'my_services' in __config__['core']['Connections']:
        __config__['core']['Connections']['my_services'] = []

    __config__['core']['Connections']['my_services'].append(node)

    __save_config__(__config__)

def remove_my_service(node):
    __config__ = {}
    with open(__os.path.join(__os.path.dirname(__file__) , 'config.json')) as json_data_file:
        __config__ = __json.load(json_data_file)

    if not 'my_services' in __config__['core']['Connections']:
        __config__['core']['Connections']['my_services'] = []
    
    if node in __config__['core']['Connections']['my_services']:
        __config__['core']['Connections']['my_services'].remove(node)

    __save_config__(__config__)

def switch_function_for_map(category, moduleName, functionName):
    global __switching_to_map__
    __switching_to_map__ = True
    mod_config_file = __os.path.join(__os.path.dirname(__file__), 'config_django.json')
    
    conf = {}

    conf = __config__['django']

    if conf:
        if not 'maps' in conf:
            conf['maps'] = {}
        in_map = '__in_map_{f}__'.format(f=functionName)

        if not moduleName in conf['maps']:
            conf['maps'][moduleName] = {}

        if not functionName in conf['maps'][moduleName]:
            conf['maps'][moduleName][functionName] = {}

        if not in_map in conf['maps'][moduleName][functionName]:
            conf['maps'][moduleName][functionName][in_map] = False
        else:
            conf['maps'][moduleName][functionName][in_map] = not conf['maps'][moduleName][functionName][in_map]

        with open(mod_config_file, 'w', encoding='utf8') as outfile:  
            __json.dump(conf, outfile, indent=4, ensure_ascii=False)

    __switching_to_map__ = False

def add_requirements_ignore(moduleName, requirementModuleName):
    __config__ = {}

    with open(__os.path.join(__os.path.dirname(__file__) , 'config.json')) as json_data_file:
        __config__ = __json.load(json_data_file)

    if not '__cant_install_requirements__' in dict(__config__['core']):
        __config__['core']['__cant_install_requirements__'] = {}

    if not moduleName in __config__['core']['__cant_install_requirements__']:
        __config__['core']['__cant_install_requirements__'][moduleName] = []

    if not str(requirementModuleName) in list(__config__['core']['__cant_install_requirements__']):
        __config__['core']['__cant_install_requirements__'][moduleName].append(str(requirementModuleName))

    __save_config__(__config__)

# API Keys

def getAPIsNames(session_id=None):
    if session_id:
        try:
            __config__['core']['__API_KEY_{sess}__'.format(sess=session_id)].keys()
        except:
            pass
    return list(__config__['core']['__API_KEY__'].keys())

# === getAPIKey ===
def getAPIKey(api_name, session_id=None):
    """
    Return an API Key registered into configuration
    
    Arguments
    ---------
        apiName : str
            
            The API Key name in String that is 
            registered into config.json
    
    Returns
    -------
        str
            
            The API Key into your config.json into 
            the apiName you selected / None
    """
    try:
        if session_id:
            try:
                sess_key = '__API_KEY_{sess}__'.format(sess=session_id)
                return __config__['core'][sess_key][api_name]
            except:
                pass
        return __config__['core']['__API_KEY__'][api_name]
    except:
        return None

# === setAPIKey ===
def setAPIKey(api_name, api_key, session_id=None):
    if session_id:
        try:
            sess_key = '__API_KEY_{sess}__'.format(sess=session_id)
            if not sess_key in __config__['core']:
                __config__['core'][sess_key] = {}
            __config__['core'][sess_key][api_name] = api_key
        except:
            __config__['core']['__API_KEY__'][api_name] = api_key
    else:
        __config__['core']['__API_KEY__'][api_name] = api_key

def loadRestAPIsFile(rest_api_file, password, session_id=None):
    with open(rest_api_file, 'r') as res:
        from hackingtools.modules.crypto.rsa import ht_rsa as r
        mod_rsa = r.StartModule()
        import hashlib
        deciphered = mod_rsa.decode(hashlib.md5(password.encode()).hexdigest(), res.read().replace('\n', ''))
        api_keys = __json.loads(deciphered)
        if session_id:
            try:
                sess_key = '__API_KEY_{sess}__'.format(sess=session_id)
                if not sess_key in __config__['core']:
                    __config__['core'][sess_key] = {}
                for k in __config__['core'][sess_key]:
                    if not k in api_keys:
                        api_keys[k] = __config__['core'][sess_key][k]
                __config__['core'][sess_key] = api_keys
            except:
                __config__['core']['__API_KEY__'] = api_keys
        else:
            __config__['core']['__API_KEY__'] = api_keys

def saveRestAPIsFile(rest_api_file, password, session_id=None):
    try:
        if session_id:
            try:
                sess_key = '__API_KEY_{sess}__'.format(sess=session_id)
                api_keys = __config__['core'][sess_key]
            except:
                api_keys = __config__['core']['__API_KEY__']
        else:
            api_keys = __config__['core']['__API_KEY__']
        data = __json.dumps(api_keys)
        from hackingtools.modules.crypto.rsa import ht_rsa as r
        mod_rsa = r.StartModule()
        import hashlib
        ciphered = mod_rsa.encode(hashlib.md5(password.encode()).hexdigest(), data)
        max_width = 64
        ciphered = '\n'.join([ciphered[y-max_width:y] for y in range(max_width, len(ciphered)+max_width,max_width)])
        with open(__os.path.join(__os.path.dirname(__file__), 'apis_files', rest_api_file), 'w') as n:
            n.write(ciphered)
        return __os.path.join(__os.path.dirname(__file__), 'apis_files', rest_api_file)
    except Exception as e:
        return str(e)

def setTelegramBotToken(token):
    __config__['core']['TelegramBot']['bot-token'] = token

# End API Keys

# Maps

def saveHostSearchedInMap(ip, location, country, info, searched_term, session_id=None):
    config_root = __config__['core']

    if session_id:
        if not session_id in config_root:
            config_root[session_id] = {}
        config_root = config_root[session_id]

    if not 'map_search' in config_root:
        config_root['map_search'] = {}

    config_root = config_root['map_search']

    config_root[ip] = {}
    
    config_root[ip]['longitude'] = location[0]
    config_root[ip]['latitude'] = location[1]

    config_root[ip]['country'] = country
    
    config_root[ip]['info'] = info
    config_root[ip]['searched_term'] = searched_term

def getSearchedHostsInMap(session_id=None):
    try:
        if session_id:
            return __config__['core'][session_id]['map_search']
        return __config__['core']['map_search']
    except:
        return {}

# End Maps

def __readFilesAuto__(djangoButtonsPool=False):
    __config__ = {}
    with open(__os.path.join(__os.path.dirname(__file__) , 'config.json')) as json_data_file:
        __config__ = __json.load(json_data_file)

    if __config__["core"]["LOAD_DJANGO_CONF"] == True:
        django = True

    if django:
        # Load the basic config for Django
        global __switching_to_map__
        if not __switching_to_map__:
            with open(__os.path.join(__os.path.dirname(__file__) , 'config_django.json')) as json_data_file_django:
                __config__['django'] = {}
                config_django = __json.load(json_data_file_django)
                for conf in config_django:
                    __config__['django'][conf] = {}
                    for django_data in config_django[conf]:
                        __config__['django'][conf][django_data] = config_django[conf][django_data] 

            # Loads the config for the modules into Django as modal forms
            categories_dir = __os.path.join(__os.path.dirname(__file__), 'config_modules_django')
            for mod in __config__['modules']:
                categories = __os.listdir(categories_dir)
                for cat in categories:
                    module_config_file = __os.path.join(categories_dir, cat, '{mod}.json'.format(mod=mod))
                    if __os.path.isfile(module_config_file):
                        with open(module_config_file) as json_data_file_django:
                            if json_data_file_django:
                                config_django = __json.load(json_data_file_django)
                                for conf in config_django:
                                    __config__['modules'][mod][conf] = config_django[conf]
                                    if djangoButtonsPool:
                                        if isinstance(__config__['modules'][mod][conf], dict):
                                            for f in __config__['modules'][mod][conf]:
                                                if '__pool_it_' in f:
                                                    __config__['modules'][mod][conf][f]['selected'] = True
    return __config__

def __djangoSwitchPoolItButtons__(checked=False):
    global __config__
    __config__ = __readFilesAuto__(djangoButtonsPool=checked)
    return __config__

# === __readConfig__ ===
def __readConfig__(django=False):
    """
    Read's all the configuration included in 
    your config.json file and inside config_modules_django directory
    recursively by your loaded modules into your config.json
    """
    global __config__
    __config__ = __readFilesAuto__()

# === __save_config__ ===
def __save_config__(new_conf, config_file='config.json'):
    """
    Save configuration passed as parameter
    to this function. This, writes into 
    your config.json
    
    Arguments
    ---------
        new_conf : List
            
            The List with the configuration you 
            want to dump onto the file
    """
    for api_name in new_conf['core']['__API_KEY__']:
        new_conf['core']['__API_KEY__'][api_name] = ''
    with open(__os.path.join(__os.path.dirname(__file__) , config_file), 'w', encoding='utf8') as outfile:  
        __json.dump(new_conf, outfile, indent=4, ensure_ascii=False)

def __cleanHtPassFiles__():
    if 'apis_files' in __os.listdir(__os.path.dirname(__file__)):
        for htpass_file in __os.listdir(__os.path.join(__os.path.dirname(__file__), 'apis_files')):
            __os.remove(__os.path.join(__os.path.dirname(__file__), 'apis_files', htpass_file))

# === __save_config__ ===
def __save_django_module_config__(new_conf, category, moduleName, functionName, is_main=True):
    """
    Save configuration passed as parameter
    to this function. This, writes into 
    your module view config ht_moduleName.json
    
    Arguments
    ---------
        new_conf : List
            
            The List with the configuration you 
            want to dump onto the file
    """
    config_file='ht_{moduleName}.json'.format(moduleName=moduleName.replace('ht_', ''))
    module_views_config_file = __os.path.join(__os.path.dirname(__file__) , 'config_modules_django', category, config_file)
    
    add_django_modal_to = 'django_form_module_function'
    if is_main:
        add_django_modal_to = 'django_form_main_function'

    __config__ = {}
    __config__['__gui_label__'] = moduleName
    __config__[add_django_modal_to] = {}

    if __os.path.isfile(module_views_config_file):
        with open(module_views_config_file, 'r', encoding='utf8') as outfile:
            if outfile:
                __config__ = __json.load(outfile)

    if not add_django_modal_to in __config__:
        __config__[add_django_modal_to] = {}
    else: # is_main - not deleted in last line
        if '_django_form_main_function_' in __config__:
            del(__config__['_django_form_main_function_'])
        if not add_django_modal_to in __config__:
            __config__[add_django_modal_to] = {}
        if '__function__' in __config__[add_django_modal_to]:
            if __config__[add_django_modal_to]['__function__'] != new_conf['__function__']:
                __config__[add_django_modal_to] = {}

    if is_main and not __config__[add_django_modal_to]:
        __config__[add_django_modal_to] = new_conf
    elif add_django_modal_to == 'django_form_module_function':
        __config__[add_django_modal_to][functionName] = new_conf

    file_path, _ = __os.path.split(module_views_config_file)
    if not __os.path.isdir(file_path):
        __os.mkdir(file_path)

    with open(module_views_config_file, 'w', encoding='utf8') as outfile:  
        __json.dump(__config__, outfile, indent=4, ensure_ascii=False)

def __regenerateConfigModulesDjango__(new_conf, category, moduleName):
    config_file='ht_{moduleName}.json'.format(moduleName=moduleName.replace('ht_', ''))
    module_views_config_file = __os.path.join(__os.path.dirname(__file__) , 'config_modules_django', category, config_file)

    __config__ = {}

    if __os.path.isfile(module_views_config_file):
        with open(module_views_config_file, 'r', encoding='utf8') as outfile:
            if outfile:
                __config__ = __json.load(outfile)

    __config__['django_form_module_function'] = new_conf
    
    with open(module_views_config_file, 'w', encoding='utf8') as outfile:  
        __json.dump(__config__, outfile, indent=4, ensure_ascii=False)

# === __createModuleTemplateConfig__ ===
def __createModuleTemplateConfig__(module_name, category):
    """
    This function creates into config.json a template
    with a simple config for trying in your GUI your 
    functions. You would see in configuration file a new 
    ht_yourmodulename key with some more data inside it.
    They are necesary when using the framework GUI for 
    trying the functions.
    
    Arguments
    ---------
        module_name : str
            
            A function name in String that is loaded 
            in your hackingtools library
    """
    module_name = 'ht_{mod}'.format(mod=module_name)

    with open(__os.path.join(__os.path.dirname(__file__) , 'config.json')) as json_data_file:
        config_tmp = __json.load(json_data_file)
        if not module_name in config_tmp['modules']:
            config_tmp['modules'][module_name] = {}
            __save_config__(config_tmp)
            
    category_dir = __os.path.join(__os.path.dirname(__file__), 'config_modules_django', category)
    if not __os.path.isdir(category_dir):
        __os.mkdir(category_dir)

    module_config_file = __os.path.join(category_dir, '{mod}.json'.format(mod=module_name))

    d = {}
    with open(module_config_file, 'r', encoding='utf8') as outfile:  
        d = __json.load(outfile)

    if not d:
        new_conf = {
            "__gui_label__" : "_MODULE_GUI_LABEL_",
            "_comment" : "Rename templates if have to use: (remove underscore) 'django_form_main_function' and 'django_form_module_function'",
            "_django_form_main_function_" : {
                "__function__" : "_FUNCTION_NAME_",
                "_HTML_FIELD_NAME_" : {
                    "__type__" : "_HTML_INPUT_TYPE_",
                    "label_desc" : "_DESCRIPTION_LABEL_",
                    "required" : "_IF_REQUIRED_",
                    "value" : "_INPUT_VALUE_",
                    "loading_text" : "_INPUT_LOADING_TEXT_",
                    "returnable_modules_functions" : {
                        "_MODULE_CALL_FOR_" : [
                            "_THAT_MODULES_FUNCTION"
                        ]
                    },
                    "options_from_function": {
                        "__CORE_OR_MODULE_NAME__": "__FUNCTION_TO_CALL__"
                    }
                }
            },
            "_django_form_module_function_" : {
                "_PUBLIC_FUNCTION_" : {
                    "__function__" : "_FUNCTION_NAME_",
                    "__async__" : False,
                    "__return__" : "_IF_RETURNS_LIKE_TEXT_",
                    "_PARAM_TO_USE_IN_VIEWS_PY_" : {
                        "__type__" : "_HTML_INPUT_TYPE_",
                        "label_desc" : "_DESCRIPTION_LABEL_",
                        "value" : "_INPUT_VALUE_",
                        "required" : "_IF_REQUIRED_",
                        "returnable_modules_functions" : {
                            "_MODULE_CALL_FOR_" : [
                                "_THAT_MODULES_FUNCTION"
                            ]
                        },
                        "options_from_function": {
                            "__CORE_OR_MODULE_NAME__": "__FUNCTION_TO_CALL__"
                        }
                    }
                }
            }
        }
        with open(module_config_file, 'w', encoding='utf8') as outfile:  
            __json.dump(new_conf, outfile, indent=4, ensure_ascii=False)

# === __look_for_changes__ ===
def __look_for_changes__(django=False):
    """
    Reloads the configuration loaded
    
    Returns
    -------
        boolean
            
            Tell's if there where any changes
    """
    config_tmp = __readFilesAuto__()

    global __config__
    if not sorted(__config__.items()) == sorted(config_tmp.items()):
        __config__ = config_tmp
        return True
    return False

__readConfig__()


from hackingtools.core import Logger, Utils, Config
if Utils.amIdjango(__name__):
	from .library.core import hackingtools as ht
else:
	import hackingtools as ht
import os

import binascii, base64

config = Config.getConfig(parentKey='modules', key='ht_rc4')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():

	def __init__(self):
		self._main_gui_func_ = ''
		self._funcArgFromFunc_ = {
			'_functionName_' : {
				'_functionParamName_' : {
					'_moduleName_' : '_functionName_' 
				}
			}
		}
		self.state = [None] * 256
		self.p = None
		self.q = None

	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_rc4'), debug_module=True)
	
	def encrypt(self, password, text):
		try:
			S = list(range(256))
			j = 0

			for i in list(range(256)):
				j = (j + S[i] + ord(password[i % len(password)])) % 256
				S[i], S[j] = S[j], S[i]

			j = 0
			y = 0
			out = []

			for char in text:
				j = (j + 1) % 256
				y = (y + S[j]) % 256
				S[j], S[y] = S[y], S[j]

				out.append(chr(ord(char) ^ S[(S[j] + S[y]) % 256]))

			return ''.join(out)
		except Exception as e:
			Logger.printMessage(str(e), is_error=True)
			return ''

	def decrypt(self, password, text):
		return self.encrypt(password, text)



import inspect as __inspect
import os as __os
from datetime import datetime as __datetime
import colorama as __colorama
from colorama import Fore as __Fore

from . import Config, Utils
config = Config.getConfig(parentKey='core', key='Logger')


import logging
logger = logging.getLogger('hackingtools')
# Prevent multiple instances
if not logger.handlers:
    logger.setLevel(logging.DEBUG)
    # create file handler which logs even debug messages
    fh = logging.FileHandler('hackingtools.log')
    fh.setLevel(logging.DEBUG)
    # create formatter and add it to the handlers
    formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
    fh.setFormatter(formatter)
    # add the handlers to the logger
    logger.addHandler(fh)

__colorama.init()

global __logs__
__logs__ = {}

global __logs_clear__
__logs_clear__ = {}

global __DEBUG_CORE_FLAG__ 
__DEBUG_CORE_FLAG__ = config['__DEBUG_CORE_FLAG__']

global __DEBUG_MODULE_FLAG__ 
__DEBUG_MODULE_FLAG__ = config['__DEBUG_MODULE_FLAG__']

global __DEBUG_USER__ 
__DEBUG_USER__ = config['__DEBUG_USER__']

if config['clear_on_load']:
    print(__colorama.ansi.clear_screen()) # Clear Screen Code

def setDebugCore(on=True):
    """Function for establising __DEBUG_CORE_FLAG__ to True/False
    
    Keyword Arguments:
        on {boolean} -- Set's True or False the Debug messages for Core Logger calls (default: {True})
    """
    global __DEBUG_CORE_FLAG__
    __DEBUG_CORE_FLAG__ = on

def setDebugModule(on=True):
    """Function for establising __DEBUG_MODULE_FLAG__ to True/False
    
    Keyword Arguments:
        on {boolean} -- Set's True or False the Debug messages for Module Logger calls (default: {True})
    """
    global __DEBUG_MODULE_FLAG__
    __DEBUG_MODULE_FLAG__ = on

def setDebug(on=True):
    """Function for establising __DEBUG_USER__ to True/False
    
    Keyword Arguments:
        on {boolean} -- Set's True or False the Debug messages for User Logger calls (default: {True})
    """
    global __DEBUG_USER__
    __DEBUG_USER__ = on

def saveLog():
    """Creates a file with the log created by the library since loaded last time.
    Output format can be changed into config.json
    """
    global __logs__
    time_now = __datetime.utcnow().strftime(config['log_save_date_format'])[:-3]
    with open('log-{time}.txt'.format(time=time_now), 'w') as f:
        for log in getLogs():
            f.write("{log}\n".format(log=log))

def getLogs():
    global __logs__
    return __logs__

def getLogsClear():
    global __logs_clear__
    return __logs_clear__

def print_and_return(msg, value, debug_module=False, debug_core=False, is_error=False, color=None):
    printMessage(message=msg, description=value, debug_core=debug_core, is_error=is_error, color=color)
    return value

def printMessage(message, description=None, debug_module=False, debug_core=False, is_error=False, is_warn=False, is_info=False, is_success=False, color=None):
    """This function prints a pretty message in console. The colors can be changed in config.json
    
    Arguments:
        message {String} -- The main message you want to show on console log
    
    Keyword Arguments:
        description {String} -- The description of the message you are showing (default: {None})
        debug_module {bool} -- Tell's if the message is been called from a module (default: {False})
        debug_core {bool} -- Tell's if the message is been called from any core file (default: {False})
        is_error {bool} -- For showing in other color because been an error (default: {False})
        color {[type]} -- A color for showing with other color (default: {None})
    """
    global __logs__
    global __logs_clear__
    filename = __inspect.stack()[1].filename
    methodName = __inspect.stack()[1].function
    
    if methodName.startswith('<'):
        methodCalledFrom = filename.split('\\')[-1]
    else:
        methodCalledFrom = '{file}.{function}()'.format(file=filename.split('\\')[-1], function=methodName)
    
    methodCalledFrom = str(__os.path.join(__os.path.split(__os.path.split(methodCalledFrom)[0])[1], __os.path.split(methodCalledFrom)[1]))

    time_now = Utils.getTime()

    colorMessage = __Fore.LIGHTMAGENTA_EX

    if color:
        colorMessage = color

    if is_error:
        colorMessage = __Fore.RED

    # Clear Log Starts Here
    successful_flag = '[SUCCESS]'
    error_flag = '[ERROR]'
    core_flag = '[CORE]'
    module_flag = '[MODULE]'
    warn_flag = '[WARN]'
    info_flag = '[INFO]' # Not used yet

    if description:
        msg = '{methodCalledFrom} - {message} - {description}'.format(methodCalledFrom=methodCalledFrom, message=message, description=description)
    else:
        msg = '{methodCalledFrom} - {message}'.format(methodCalledFrom=methodCalledFrom, message=message)
    
    if is_error:
        logger.critical(msg)
    elif is_info:
        logger.info(msg)
    elif is_success:
        logger.debug(msg)
    elif is_warn:
        logger.warning(msg)
    else:
        logger.debug(msg)

    if (color == __Fore.YELLOW or is_warn):
        msg = '{e} - {m}'.format(e=warn_flag, m=msg)
    elif (color == __Fore.GREEN or is_success):
        msg = '{e} - {m}'.format(e=successful_flag, m=msg)
    elif (color == __Fore.BLUE or is_info):
        msg = '{e} - {m}'.format(e=info_flag, m=msg)
    elif is_error:
        msg = '{e} - {m}'.format(e=error_flag, m=msg)
    elif debug_module:
        msg = '{e} - {m}'.format(e=module_flag, m=msg)
    elif debug_core:
        msg = '{e} - {m}'.format(e=core_flag, m=msg)

    if description:
        __logs__[time_now] = msg
    else:
        __logs__[time_now] = msg

    temp_logs_clear = {}

    for x in __logs__:

        exists = False

        for temp in temp_logs_clear:
            if temp_logs_clear[temp]['msg'] == __logs__[x]:
                temp_logs_clear[temp]['count'] += 1
                exists = True

        if not exists:
            temp_logs_clear[x] = {}
            temp_logs_clear[x]['msg'] = __logs__[x]
            temp_logs_clear[x]['count'] = 1

    __logs_clear__ = {}

    for timeLog in temp_logs_clear:
        if temp_logs_clear[timeLog]['count'] > 1:
            __logs_clear__[timeLog] = '({n}) - {m}'.format(n=temp_logs_clear[timeLog]['count'], m=temp_logs_clear[timeLog]['msg'])
        else:
            __logs_clear__[timeLog] = temp_logs_clear[timeLog]['msg']
    # Clear Log Ends Here

    if ((not debug_module == True) and (not debug_core == True) and (__DEBUG_USER__ == True)) or ((debug_module == True) and (__DEBUG_MODULE_FLAG__ == True)) or ((debug_core == True) and (__DEBUG_CORE_FLAG__ == True)):
        if description:
            print('{timeColorStart}[{time}]{timeColorEnd} - {methodCalledFromColorStart}{methodCalledFrom}{methodCalledFromColorEnd} - {messageColorStart}{message}{messageColorEnd} - {description}'.format(
                timeColorStart=__Fore.BLUE, 
                timeColorEnd=__Fore.WHITE, 
                time=time_now,
                methodCalledFromColorStart=__Fore.GREEN, 
                methodCalledFrom=methodCalledFrom,
                methodCalledFromColorEnd=__Fore.WHITE,
                messageColorStart=colorMessage,
                message=message,
                messageColorEnd=__Fore.WHITE,
                description=description))
        else:
            print('{timeColorStart}[{time}]{timeColorEnd} - {methodCalledFromColorStart}{methodCalledFrom}{methodCalledFromColorEnd} - {messageColorStart}{message}{messageColorEnd}'.format(
                timeColorStart=__Fore.BLUE, 
                timeColorEnd=__Fore.WHITE, 
                time=time_now,
                methodCalledFromColorStart=__Fore.GREEN, 
                methodCalledFrom=methodCalledFrom,
                methodCalledFromColorEnd=__Fore.WHITE,
                messageColorStart=colorMessage,
                message=message,
                messageColorEnd=__Fore.WHITE))


import binascii

from Crypto import Random
from Crypto.PublicKey import RSA

import requests
from flask import Flask, jsonify, request, render_template

from .Objects import Transaction

app = Flask(__name__)

@app.route('/')
def index():
	return render_template('./index.html')

@app.route('/make/transaction')
def make_transaction():
    return render_template('./make_transaction.html')

@app.route('/view/transactions')
def view_transaction():
    return render_template('./view_transactions.html')

@app.route('/wallet/new', methods=['GET'])
def new_wallet():
	random_gen = Random.new().read
	private_key = RSA.generate(1024, random_gen)
	public_key = private_key.publickey()
	response = {
		'private_key': binascii.hexlify(private_key.exportKey(format='DER')).decode('ascii'),
		'public_key': binascii.hexlify(public_key.exportKey(format='DER')).decode('ascii')
	}

	return jsonify(response), 200

@app.route('/generate/transaction', methods=['POST'])
def generate_transaction():
	
	sender_address = request.form['sender_address']
	sender_private_key = request.form['sender_private_key']
	recipient_address = request.form['recipient_address']
	value = request.form['amount']

	transaction = Transaction(sender_address, sender_private_key, recipient_address, value)

	response = {'transaction': transaction.to_dict(), 'signature': transaction.sign_transaction()}

	return jsonify(response), 200


app.run(host='0.0.0.0', port='6767')


from hackingtools.core import Logger, Utils, Config
if Utils.amIdjango(__name__):
	from .library.core import hackingtools as ht
else:
	import hackingtools as ht
import os, time

from scapy.all import IP, TCP, UDP, sr, RandShort

config = Config.getConfig(parentKey='modules', key='ht_scapy')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():

	def __init__(self):
		self._main_gui_func_ = ''

	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_scapy'), debug_module=True)

	def __trace__(self, domain_or_ip):
		return sr( IP( dst=domain_or_ip, ttl=(1,6),id=RandShort() ) / TCP(flags=0x2), verbose=True )

	def traceroute(self, domain_or_ip):
		data = []
		
		ht.worker('scapy-traceroute', 'ht.getModule("scapy").__trace__', args=((domain_or_ip),), loop=False, timeout=5)

		wor = ht.getWorker('scapy-traceroute')[0]

		while wor.is_alive():
			time.sleep(1)

		ans, _ = wor.getLastResponse()

		for snd, rcv in ans:
			data.append( { 'ttl-sent' : snd.ttl, 'rcv-src' : rcv.src, 'tcp' : isinstance(rcv.payload, TCP) } )


		return data


from hackingtools.core import Logger, Config
from hackingtools.core.Objects import CookieSessionManager, InstagramEndPoints, Account, Comment, Media, Location, UserStories, Story, Tag, TwoStepConsoleVerification
from hackingtools.core.Exceptions import NotFoundException, AnyException, AuthException
import hackingtools as ht

import os
import time, requests, re, json, hashlib, random
from slugify import slugify

config = Config.getConfig(parentKey='modules', key='ht_instagram')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

HTTP_NOT_FOUND = 404
HTTP_OK = 200
HTTP_FORBIDDEN = 403
HTTP_BAD_REQUEST = 400

MAX_COMMENTS_PER_REQUEST = 300
MAX_LIKES_PER_REQUEST = 50

# 30 mins time limit on operations that require multiple self.__req
PAGING_TIME_LIMIT_SEC = 1800
PAGING_DELAY_MINIMUM_MICROSEC = 1000000  # 1 sec min delay to simulate browser
PAGING_DELAY_MAXIMUM_MICROSEC = 3000000  # 3 sec max delay to simulate browser

instance_cache = None

class StartModule():

    def __init__(self):
        self._main_gui_func_ = 'getAccountByUsername'
        self.__req = requests.session()
        self.paging_time_limit_sec = PAGING_TIME_LIMIT_SEC
        self.paging_delay_minimum_microsec = PAGING_DELAY_MINIMUM_MICROSEC
        self.paging_delay_maximum_microsec = PAGING_DELAY_MAXIMUM_MICROSEC

        self.session_username = None
        self.session_password = None
        self.user_session = None
        self.rhx_gis = None
        self.sleep_between_requests = 0
        self.user_agent = 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_4) ' \
                          'AppleWebKit/537.36 (KHTML, like Gecko) ' \
                          'Chrome/66.0.3359.139 Safari/537.36'
        
    def help(self):
        return ht.getFunctionsNamesFromModule('ht_instagram')

    def getAccountByUsername(self, username):
        try:
            return self.__get_account__(username).get_data()
        except Exception as e:
            return str(e)

    def __with_credentials__(self, username, password, session_folder=None):
        """
        param string username
        param string password
        param null sessionFolder
        return Instagram
        """
        instance_cache = None

        if not session_folder:
            cwd = os.getcwd()
            session_folder = cwd + os.path.sep + 'sessions' + os.path.sep

        if isinstance(session_folder, str):

            instance_cache = CookieSessionManager(session_folder, '{n}.txt'.format(n=slugify(username)))

        else:
            instance_cache = session_folder

        instance_cache.empty_saved_cookies()


        self.session_username = username
        self.session_password = password

    def __set_proxies__(self, proxy):
        if proxy and isinstance(proxy, dict):
            self.__req.proxies = proxy

    def __disable_verify__(self):
        self.__req.verify = False

    def __disable_proxies__(self):
        self.__req.proxies = {}

    def __get_user_agent__(self):
        return self.user_agent

    def __set_user_agent__(self, user_agent):
        self.user_agent = user_agent

    @staticmethod
    def __set_account_medias_request_count__(count):
        """
        Set how many media objects should be retrieved in a single request
        param int count
        """
        InstagramEndPoints().request_media_count = count

    def __get_account_by_id__(self, id):
        """
        :param id: account id
        :return: Account
        """
        username = self.__get_username_by_id__(id)
        return self.__get_account__(username)

    def __get_username_by_id__(self, id):
        """
        :param id: account id
        :return: username string from response
        """
        time.sleep(self.sleep_between_requests)
        response = self.__req.get(InstagramEndPoints().get_account_json_private_info_link_by_account_id(id), headers=self.__generate_headers__(self.user_session))

        if HTTP_NOT_FOUND == response.status_code:
            raise NotFoundException('Failed to fetch account with given id')

        if HTTP_OK != response.status_code:
            raise AnyException.default(response.text, response.status_code)

        json_response = response.json()
        if not json_response:
            raise AnyException('Response does not JSON')

        if json_response['status'] != 'ok':
            message = json_response['message'] if ('message' in json_response.keys()) else 'Unknown Error'
            raise AnyException(message)

        return json_response['user']['username']

    def __generate_headers__(self, session, gis_token=None):
        """
        :param session: user session dict
        :param gis_token: a token used to be verified by instagram in header
        :return: header dict
        """
        headers = {}
        if session is not None:
            cookies = ''

            for key in session.keys():
                cookies += f"{key}={session[key]}; "

            csrf = session['x-csrftoken'] if session['csrftoken'] is None else \
                session['csrftoken']

            headers = {
                'cookie': cookies,
                'referer': InstagramEndPoints().BASE_URL + '/',
                'x-csrftoken': csrf
            }

        if self.user_agent is not None:
            headers['user-agent'] = self.user_agent

            if gis_token is not None:
                headers['x-instagram-gis'] = gis_token

        return headers

    def __generate_gis_token__(self, variables):
        """
        :param variables: a dict used to  generate_gis_token
        :return: a token used to be verified by instagram
        """
        rhx_gis = self.__get_rhx_gis__() if self.__get_rhx_gis__() is not None else 'NULL'
        string_to_hash = ':'.join([rhx_gis, json.dumps(variables, separators=(',', ':')) if isinstance(variables, dict) else variables])
        return hashlib.md5(string_to_hash.encode('utf-8')).hexdigest()

    def __get_rhx_gis__(self):
        """
        :return: a string to generate gis_token
        """
        if self.rhx_gis is None:
            try:
                shared_data = self.__get_shared_data_from_page__()
            except Exception as _:
                raise AnyException('Could not extract gis from page')

            if 'rhx_gis' in shared_data.keys():
                self.rhx_gis = shared_data['rhx_gis']
            else:
                self.rhx_gis = None

        return self.rhx_gis

    def __get_mid__(self):
        """manually fetches the machine id from graphQL"""
        time.sleep(self.sleep_between_requests)
        response = self.__req.get('https://www.instagram.com/web/__mid/')

        if response.status_code != HTTP_OK:
            raise AnyException.default(response.text, response.status_code)

        return response.text

    def __get_shared_data_from_page__(self, url=InstagramEndPoints().BASE_URL):
        """
        :param url: the requested url
        :return: a dict extract from page
        """
        url = url.rstrip('/') + '/'
        time.sleep(self.sleep_between_requests)
        response = self.__req.get(url, headers=self.__generate_headers__(self.user_session))

        if HTTP_NOT_FOUND == response.status_code:
            raise NotFoundException(f"Page {url} not found")

        if not HTTP_OK == response.status_code:
            raise AnyException.default(response.text, response.status_code)

        return self.__extract_shared_data_from_body__(response.text)

    @staticmethod
    def __extract_shared_data_from_body__(body):
        """
        :param body: html string from a page
        :return: a dict extract from page
        """
        array = re.findall(r'_sharedData = .*?;</script>', body)
        if len(array) > 0:
            raw_json = array[0][len("_sharedData ="):-len(";</script>")]

            return json.loads(raw_json)

        return None

    def __search_tags_by_tag_name__(self, tag):
        """
        :param tag: tag string
        :return: list of Tag
        """
        # TODO: Add tests and auth
        time.sleep(self.sleep_between_requests)
        response = self.__req.get(InstagramEndPoints().get_general_search_json_link(tag))

        if HTTP_NOT_FOUND == response.status_code:
            raise NotFoundException(
                'Account with given username does not exist.')

        if not HTTP_OK == response.status_code:
            raise AnyException.default(response.text,
                                             response.status_code)

        json_response = response.json()

        try:
            status = json_response['status']
            if status != 'ok':
                raise AnyException(
                    'Response code is not equal 200. '
                    'Something went wrong. Please report issue.')
        except KeyError:
            raise AnyException('Response code is not equal 200. Something went wrong. Please report issue.')

        try:
            hashtags_raw = json_response['hashtags']
            if len(hashtags_raw) == 0:
                return []
        except KeyError:
            return []

        hashtags = []
        for json_hashtag in hashtags_raw:
            hashtags.append(Tag(json_hashtag['hashtag']))

        return hashtags

    def __get_medias__(self, username, count=20, maxId=''):
        """
        :param username: instagram username
        :param count: the number of how many media you want to get
        :param maxId: used to paginate
        :return: list of Media
        """
        account = self.__get_account__(username)
        return self.__get_medias_by_user_id__(account.identifier, count, maxId)

    def __get_medias_by_code__(self, media_code):
        """
        :param media_code: media code
        :return: Media
        """
        url = InstagramEndPoints().get_media_page_link(media_code)
        return self.__get_media_by_url__(url)

    def __get_medias_by_user_id__(self, id, count=12, max_id=''):
        """
        :param id: instagram account id
        :param count: the number of how many media you want to get
        :param max_id: used to paginate
        :return: list of Media
        """
        index = 0
        medias = []
        is_more_available = True

        while index < count and is_more_available:

            variables = {
                'id': str(id),
                'first': str(count),
                'after': str(max_id)
            }

            headers = self.__generate_headers__(self.user_session,
                                            self.__generate_gis_token__(
                                                variables))

            time.sleep(self.sleep_between_requests)
            response = self.__req.get(
                InstagramEndPoints().get_account_medias_json_link(variables),
                headers=headers)

            if not HTTP_OK == response.status_code:
                raise AnyException.default(response.text,
                                                 response.status_code)

            arr = json.loads(response.text)

            try:
                nodes = arr['data']['user']['edge_owner_to_timeline_media'][
                    'edges']
            except KeyError:
                return {}

            for mediaArray in nodes:
                if index == count:
                    return medias

                media = Media(mediaArray['node'])
                medias.append(media)
                index += 1

            if not nodes or nodes == '':
                return medias

            max_id = \
                arr['data']['user']['edge_owner_to_timeline_media'][
                    'page_info'][
                    'end_cursor']
            is_more_available = \
                arr['data']['user']['edge_owner_to_timeline_media'][
                    'page_info'][
                    'has_next_page']

        return medias

    def __get_media_by_id__(self, media_id):
        """
        :param media_id: media id
        :return: list of Media
        """
        media_link = Media.get_link_from_id(media_id)
        return self.__get_media_by_url__(media_link)

    def __get_media_by_url__(self, media_url):
        """
        :param media_url: media url
        :return: Media
        """
        url_regex = r'http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+'

        if len(re.findall(url_regex, media_url)) <= 0:
            raise ValueError('Malformed media url')

        url = media_url.rstrip('/') + '/?__a=1'
        time.sleep(self.sleep_between_requests)
        response = self.__req.get(url, headers=self.__generate_headers__(
            self.user_session))

        if HTTP_NOT_FOUND == response.status_code:
            raise NotFoundException(
                'Media with given code does not exist or account is private.')

        if HTTP_OK != response.status_code:
            raise AnyException.default(response.text,
                                             response.status_code)

        media_array = response.json()
        try:
            media_in_json = media_array['graphql']['shortcode_media']
        except KeyError:
            raise AnyException('Media with this code does not exist')

        return Media(media_in_json)

    def __get_medias_from_feed__(self, username, count=20):
        """
        :param username: instagram username
        :param count: the number of how many media you want to get
        :return: list of Media
        """
        medias = []
        index = 0
        time.sleep(self.sleep_between_requests)
        response = self.__req.get(InstagramEndPoints().get_account_json_link(username),
                                  headers=self.__generate_headers__(
                                      self.user_session))

        if HTTP_NOT_FOUND == response.status_code:
            raise NotFoundException(
                'Account with given username does not exist.')

        if HTTP_OK != response.status_code:
            raise AnyException.default(response.text,
                                             response.status_code)

        user_array = response.json()

        try:
            user = user_array['graphql']['user']
        except KeyError:
            raise NotFoundException(
                'Account with this username does not exist')

        try:
            nodes = user['edge_owner_to_timeline_media']['edges']
            if len(nodes) == 0:
                return []
        except Exception:
            return []

        for media_array in nodes:
            if index == count:
                return medias
            medias.append(Media(media_array['node']))
            index += 1

        return medias

    def __get_medias_by_tag__(self, tag, count=12, max_id='', min_timestamp=None):
        """
        :param tag: tag string
        :param count: the number of how many media you want to get
        :param max_id: used to paginate
        :param min_timestamp: limit the time you want to start from
        :return: list of Media
        """
        index = 0
        medias = []
        media_ids = []
        has_next_page = True
        while index < count and has_next_page:

            time.sleep(self.sleep_between_requests)
            response = self.__req.get(
                InstagramEndPoints().get_medias_json_by_tag_link(tag, max_id),
                headers=self.__generate_headers__(self.user_session))

            if response.status_code != HTTP_OK:
                raise AnyException.default(response.text,
                                                 response.status_code)

            arr = response.json()

            try:
                arr['graphql']['hashtag']['edge_hashtag_to_media']['count']
            except KeyError:
                return []

            nodes = arr['graphql']['hashtag']['edge_hashtag_to_media']['edges']
            for media_array in nodes:
                if index == count:
                    return medias
                media = Media(media_array['node'])
                if media.identifier in media_ids:
                    return medias

                if min_timestamp is not None \
                        and media.created_time < min_timestamp:
                    return medias

                media_ids.append(media.identifier)
                medias.append(media)
                index += 1

            if len(nodes) == 0:
                return medias

            max_id = \
                arr['graphql']['hashtag']['edge_hashtag_to_media']['page_info'][
                    'end_cursor']
            has_next_page = \
                arr['graphql']['hashtag']['edge_hashtag_to_media']['page_info'][
                    'has_next_page']

        return medias

    def __get_medias_by_location_id__(self, facebook_location_id, count=24,
                                  max_id=''):
        """
        :param facebook_location_id: facebook location id
        :param count: the number of how many media you want to get
        :param max_id: used to paginate
        :return: list of Media
        """
        index = 0
        medias = []
        has_next_page = True

        while index < count and has_next_page:

            time.sleep(self.sleep_between_requests)
            response = self.__req.get(
                InstagramEndPoints().get_medias_json_by_location_id_link(
                    facebook_location_id, max_id),
                headers=self.__generate_headers__(self.user_session))

            if response.status_code != HTTP_OK:
                raise AnyException.default(response.text,
                                                 response.status_code)

            arr = response.json()

            nodes = arr['graphql']['location']['edge_location_to_media'][
                'edges']
            for media_array in nodes:
                if index == count:
                    return medias

                medias.append(Media(media_array['node']))
                index += 1

            if len(nodes) == 0:
                return medias

            has_next_page = \
                arr['graphql']['location']['edge_location_to_media'][
                    'page_info'][
                    'has_next_page']
            max_id = \
                arr['graphql']['location']['edge_location_to_media'][
                    'page_info'][
                    'end_cursor']

        return medias

    def __get_current_top_medias_by_tag_name__(self, tag_name):
        """
        :param tag_name: tag string
        :return: list of the top Media
        """
        time.sleep(self.sleep_between_requests)
        response = self.__req.get(
            InstagramEndPoints().get_medias_json_by_tag_link(tag_name, ''),
            headers=self.__generate_headers__(self.user_session))

        if response.status_code == HTTP_NOT_FOUND:
            raise NotFoundException(
                'Account with given username does not exist.')

        if response.status_code is not HTTP_OK:
            raise AnyException.default(response.text,
                                             response.status_code)

        json_response = response.json()
        medias = []

        nodes = \
            json_response['graphql']['hashtag']['edge_hashtag_to_top_posts'][
                'edges']

        for media_array in nodes:
            medias.append(Media(media_array['node']))

        return medias

    def __get_current_top_medias_by_location_id__(self, facebook_location_id):
        """
        :param facebook_location_id: facebook location id
        :return: list of the top Media
        """
        time.sleep(self.sleep_between_requests)
        response = self.__req.get(
            InstagramEndPoints().get_medias_json_by_location_id_link(facebook_location_id),
            headers=self.__generate_headers__(self.user_session))
        if response.status_code == HTTP_NOT_FOUND:
            raise NotFoundException(
                "Location with this id doesn't exist")

        if response.status_code != HTTP_OK:
            raise AnyException.default(response.text,
                                             response.status_code)

        json_response = response.json()

        nodes = \
            json_response['graphql']['location']['edge_location_to_top_posts'][
                'edges']
        medias = []

        for media_array in nodes:
            medias.append(Media(media_array['node']))

        return medias

    def __get_paginate_medias__(self, username, max_id=''):
        """
        :param username: instagram user name
        :param max_id: used to paginate next time
        :return: dict that contains Media list, maxId, hasNextPage
        """
        account = self.__get_account__(username)
        has_next_page = True
        medias = []

        to_return = {
            'medias': medias,
            'maxId': max_id,
            'hasNextPage': has_next_page,
        }

        variables = json.dumps({
            'id': str(account.identifier),
            'first': str(InstagramEndPoints().request_media_count),
            'after': str(max_id)
        }, separators=(',', ':'))

        time.sleep(self.sleep_between_requests)
        response = self.__req.get(
            InstagramEndPoints().get_account_medias_json_link(variables),
            headers=self.__generate_headers__(self.user_session,
                                          self.__generate_gis_token__(variables))
        )

        if not HTTP_OK == response.status_code:
            raise AnyException.default(response.text,
                                             response.status_code)

        arr = response.json()

        try:
            nodes = arr['data']['user']['edge_owner_to_timeline_media']['edges']
        except KeyError:
            return to_return

        for mediaArray in nodes:
            medias.append(Media(mediaArray['node']))

        max_id = \
            arr['data']['user']['edge_owner_to_timeline_media']['page_info'][
                'end_cursor']
        has_next_page = \
            arr['data']['user']['edge_owner_to_timeline_media']['page_info'][
                'has_next_page']

        to_return = {
            'medias': medias,
            'maxId': max_id,
            'hasNextPage': has_next_page,
        }

        return to_return

    def __get_paginate_medias_by_tag__(self, tag, max_id=''):
        """
        :param tag: tag name
        :param max_id: used to paginate next time
        :return: dict that contains Media list, maxId, hasNextPage
        """
        has_next_page = True
        medias = []

        to_return = {
            'medias': medias,
            'maxId': max_id,
            'hasNextPage': has_next_page,
        }

        time.sleep(self.sleep_between_requests)
        response = self.__req.get(
            InstagramEndPoints().get_medias_json_by_tag_link(tag, max_id),
            headers=self.__generate_headers__(self.user_session))

        if response.status_code != HTTP_OK:
            raise AnyException.default(response.text,
                                             response.status_code)

        arr = response.json()

        try:
            nodes = arr['graphql']['hashtag']['edge_hashtag_to_media']['edges']
        except KeyError:
            return to_return

        for media_array in nodes:
            medias.append(Media(media_array['node']))

        max_id = \
            arr['graphql']['hashtag']['edge_hashtag_to_media']['page_info'][
                'end_cursor']
        has_next_page = \
            arr['graphql']['hashtag']['edge_hashtag_to_media']['page_info'][
                'has_next_page']
        try:
            media_count = arr['graphql']['hashtag']['edge_hashtag_to_media'][
                'count']
        except KeyError:
            return to_return

        to_return = {
            'medias': medias,
            'count': media_count,
            'maxId': max_id,
            'hasNextPage': has_next_page,
        }

        return to_return

    def __get_location_by_id__(self, facebook_location_id):
        """
        :param facebook_location_id: facebook location id
        :return: Location
        """
        time.sleep(self.sleep_between_requests)
        response = self.__req.get(
            InstagramEndPoints().get_medias_json_by_location_id_link(facebook_location_id),
            headers=self.__generate_headers__(self.user_session))

        if response.status_code == HTTP_NOT_FOUND:
            raise NotFoundException(
                'Location with this id doesn\'t exist')

        if response.status_code != HTTP_OK:
            raise AnyException.default(response.text,
                                             response.status_code)

        json_response = response.json()

        return Location(json_response['graphql']['location'])

    def __get_media_likes_by_code__(self, code, count=10, max_id=None):
        """
        :param code:
        :param count:
        :param max_id:
        :return:
        """

        remain = count
        likes = []
        index = 0
        has_previous = True

        #TODO: $index < $count (bug index getting to high since max_likes_per_request gets sometimes changed by instagram)

        while (has_previous and index < count):
            if (remain > MAX_LIKES_PER_REQUEST):
                number_of_likes_to_receive = MAX_LIKES_PER_REQUEST
                remain -= MAX_LIKES_PER_REQUEST
                index += MAX_LIKES_PER_REQUEST
            else:
                number_of_likes_to_receive = remain
                index += remain
                remain = 0


            variables = {
                "shortcode": str(code),
                "first": str(number_of_likes_to_receive),
                "after": '' if not max_id else max_id
            }

            time.sleep(self.sleep_between_requests)

            response = self.__req.get(
                InstagramEndPoints().get_last_likes_by_code(variables),
                headers=self.__generate_headers__(self.user_session))

            if not response.status_code == HTTP_OK:
                raise AnyException.default(response.text,response.status_code)

            jsonResponse = response.json()

            nodes = jsonResponse['data']['shortcode_media']['edge_liked_by']['edges']

            for likesArray in nodes:

                like = Account(likesArray['node'])
                likes.append(like)


            has_previous = jsonResponse['data']['shortcode_media']['edge_liked_by']['page_info']['has_next_page']
            number_of_likes = jsonResponse['data']['shortcode_media']['edge_liked_by']['count']
            if count > number_of_likes:
                count = number_of_likes

            if len(nodes) == 0:
                data = {}
                data['next_page'] = max_id
                data['accounts'] = likes

                return data

            max_id = jsonResponse['data']['shortcode_media']['edge_liked_by']['page_info']['end_cursor']

        data = {}
        data['next_page'] = max_id
        data['accounts'] = likes

        return data

    def __get_followers__(self, account_id, count=20, page_size=20, end_cursor='',
                      delayed=True):

        """
        :param account_id:
        :param count:
        :param page_size:
        :param end_cursor:
        :param delayed:
        :return:
        """
        # TODO set time limit
        # if ($delayed) {
        #     set_time_limit($this->pagingTimeLimitSec);
        # }

        index = 0
        accounts = []

        next_page = end_cursor

        if count < page_size:
            raise AnyException(
                'Count must be greater than or equal to page size.')

        while True:
            time.sleep(self.sleep_between_requests)

            variables = {
                'id': str(account_id),
                'first': str(count),
                'after': next_page
            }

            headers = self.__generate_headers__(self.user_session)

            response = self.__req.get(
                InstagramEndPoints().get_followers_json_link(variables),
                headers=headers)

            if not response.status_code == HTTP_OK:
                raise AnyException.default(response.text,
                                                 response.status_code)

            jsonResponse = response.json()

            if jsonResponse['data']['user']['edge_followed_by']['count'] == 0:
                return accounts

            edgesArray = jsonResponse['data']['user']['edge_followed_by'][
                'edges']
            if len(edgesArray) == 0:
                AnyException(
                    f'Failed to get followers of account id {account_id}.'
                    f' The account is private.',
                    HTTP_FORBIDDEN)

            pageInfo = jsonResponse['data']['user']['edge_followed_by'][
                'page_info']
            if pageInfo['has_next_page']:
                next_page = pageInfo['end_cursor']

            for edge in edgesArray:

                accounts.append(Account(edge['node']))
                index += 1

                if index >= count:
                    #since break 2 not in python, looking for better solution since duplicate code
                    data = {}
                    data['next_page'] = next_page
                    data['accounts'] = accounts

                    return data

            #must be below here
            if not pageInfo['has_next_page']:
                break

            if delayed != None:
                # Random wait between 1 and 3 sec to mimic browser
                microsec = random.uniform(1.0, 3.0)
                time.sleep(microsec)

        data = {}
        data['next_page'] = next_page
        data['accounts'] = accounts

        return data

    def __get_following__(self, account_id, count=20, page_size=20, end_cursor='',
                      delayed=True):
        """
        :param account_id:
        :param count:
        :param page_size:
        :param end_cursor:
        :param delayed:
        :return:
        """

        #TODO
        #     if ($delayed) {
        #         set_time_limit($this->pagingTimeLimitSec);
        #     }

        index = 0
        accounts = []

        next_page = end_cursor

        if count < page_size:
            raise AnyException('Count must be greater than or equal to page size.')

        while True:

            variables = {
                'id': str(account_id),
                'first': str(count),
                'after': next_page
            }

            headers = self.__generate_headers__(self.user_session)


            response = self.__req.get(
                InstagramEndPoints().get_following_json_link(variables),
                headers=headers)

            if not response.status_code == HTTP_OK:
                raise AnyException.default(response.text,response.status_code)

            jsonResponse = response.json()
            if jsonResponse['data']['user']['edge_follow']['count'] == 0:
                return accounts

            edgesArray = jsonResponse['data']['user']['edge_follow'][
                'edges']

            if len(edgesArray) == 0:
                raise AnyException(
                    f'Failed to get follows of account id {account_id}.'
                    f' The account is private.',
                    HTTP_FORBIDDEN)

            pageInfo = jsonResponse['data']['user']['edge_follow']['page_info']
            if pageInfo['has_next_page']:
                next_page = pageInfo['end_cursor']

            for edge in edgesArray:
                accounts.append(Account(edge['node']))
                index += 1
                if index >= count:
                    #since no break 2, looking for better solution since duplicate code
                    data = {}
                    data['next_page'] = next_page
                    data['accounts'] = accounts

                    return data

            #must be below here
            if not pageInfo['has_next_page']:
                break

            if delayed != None:
                # Random wait between 1 and 3 sec to mimic browser
                microsec = random.uniform(1.0, 3.0)
                time.sleep(microsec)

        data = {}
        data['next_page'] = next_page
        data['accounts'] = accounts

        return data

    def __get_media_comments_by_id__(self, media_id, count=10, max_id=None):
        """
        :param media_id: media id
        :param count: the number of how many comments you want to get
        :param max_id: used to paginate
        :return: Comment List
        """
        code = Media.get_code_from_id(media_id)
        return self.__get_media_comments_by_code__(code, count, max_id)

    def __get_media_comments_by_code__(self, code, count=10, max_id=''):

        """
        :param code: media code
        :param count: the number of how many comments you want to get
        :param max_id: used to paginate
        :return: Comment List
        """

        comments = []
        index = 0
        has_previous = True

        while has_previous and index < count:
            number_of_comments_to_receive = 0
            if count - index > MAX_COMMENTS_PER_REQUEST:
                number_of_comments_to_receive = MAX_COMMENTS_PER_REQUEST
            else:
                number_of_comments_to_receive = count - index

            variables = {
                "shortcode": str(code),
                "first": str(number_of_comments_to_receive),
                "after": '' if not max_id else max_id
            }

            comments_url = InstagramEndPoints().get_comments_before_comments_id_by_code(
                variables)

            time.sleep(self.sleep_between_requests)
            response = self.__req.get(comments_url,
                                      headers=self.__generate_headers__(
                                          self.user_session,
                                          self.__generate_gis_token__(variables)))

            if not response.status_code == HTTP_OK:
                raise AnyException.default(response.text,
                                                 response.status_code)

            jsonResponse = response.json()

            nodes = jsonResponse['data']['shortcode_media']['edge_media_to_parent_comment']['edges']

            for commentArray in nodes:
                comment = Comment(commentArray['node'])
                comments.append(comment)
                index += 1

            has_previous = jsonResponse['data']['shortcode_media']['edge_media_to_parent_comment']['page_info']['has_next_page']

            number_of_comments = jsonResponse['data']['shortcode_media']['edge_media_to_parent_comment']['count']
            if count > number_of_comments:
                count = number_of_comments

            max_id = jsonResponse['data']['shortcode_media']['edge_media_to_parent_comment']['page_info']['end_cursor']

            if len(nodes) == 0:
                break


        data = {}
        data['next_page'] = max_id
        data['comments'] = comments
        return data

    def __get_account__(self, username):
        """
        :param username: username
        :return: Account
        """
        time.sleep(self.sleep_between_requests)
        response = self.__req.get(InstagramEndPoints().get_account_page_link(username), headers=self.__generate_headers__(self.user_session))

        if HTTP_NOT_FOUND == response.status_code:
            raise NotFoundException('Account with given username does not exist.')

        if HTTP_OK != response.status_code:
            raise AnyException.default(response.text, response.status_code)

        user_array = self.__extract_shared_data_from_body__(response.text)

        if user_array['entry_data']['ProfilePage'][0]['graphql']['user'] is None:
            raise NotFoundException('Account with this username does not exist')

        return Account(user_array['entry_data']['ProfilePage'][0]['graphql']['user'])

    def __get_stories__(self, reel_ids=None):
        """
        :param reel_ids: reel ids
        :return: UserStories List
        """
        variables = {'precomposed_overlay': False, 'reel_ids': []}

        if reel_ids is None or len(reel_ids) == 0:
            time.sleep(self.sleep_between_requests)
            response = self.__req.get(InstagramEndPoints().get_user_stories_link(),
                                      headers=self.__generate_headers__(
                                          self.user_session))

            if not HTTP_OK == response.status_code:
                raise AnyException.default(response.text,
                                                 response.status_code)

            json_response = response.json()

            try:
                edges = json_response['data']['user']['feed_reels_tray'][
                    'edge_reels_tray_to_reel']['edges']
            except KeyError:
                return []

            for edge in edges:
                variables['reel_ids'].append(edge['node']['id'])

        else:
            variables['reel_ids'] = reel_ids

        time.sleep(self.sleep_between_requests)
        response = self.__req.get(InstagramEndPoints().get_stories_link(variables),
                                  headers=self.__generate_headers__(
                                      self.user_session))

        if not HTTP_OK == response.status_code:
            raise AnyException.default(response.text,
                                             response.status_code)

        json_response = response.json()

        try:
            reels_media = json_response['data']['reels_media']
            if len(reels_media) == 0:
                return []
        except KeyError:
            return []

        stories = []
        for user in reels_media:
            user_stories = UserStories()

            user_stories.owner = Account(user['user'])
            for item in user['items']:
                story = Story(item)
                user_stories.stories.append(story)

            stories.append(user_stories)
        return stories

    def __search_accounts_by_username__(self, username):
        """
        :param username: user name
        :return: Account List
        """
        time.sleep(self.sleep_between_requests)
        response = self.__req.get(
            InstagramEndPoints().get_general_search_json_link(username),
            headers=self.__generate_headers__(self.user_session))

        if HTTP_NOT_FOUND == response.status_code:
            raise NotFoundException(
                'Account with given username does not exist.')

        if not HTTP_OK == response.status_code:
            raise AnyException.default(response.text,
                                             response.status_code)

        json_response = response.json()

        try:
            status = json_response['status']
            if not status == 'ok':
                raise AnyException(
                    'Response code is not equal 200.'
                    ' Something went wrong. Please report issue.')
        except KeyError:
            raise AnyException(
                'Response code is not equal 200.'
                ' Something went wrong. Please report issue.')

        try:
            users = json_response['users']
            if len(users) == 0:
                return []
        except KeyError:
            return []

        accounts = []
        for json_account in json_response['users']:
            accounts.append(Account(json_account['user']))

        return accounts

    # TODO not optimal separate http call after getMedia
    def __get_media_tagged_users_by_code__(self, code):
        """
        :param code: media short code
        :return: list contains tagged_users dict
        """
        url = InstagramEndPoints().get_media_json_link(code)

        time.sleep(self.sleep_between_requests)
        response = self.__req.get(url, headers=self.__generate_headers__(
            self.user_session))

        if not HTTP_OK == response.status_code:
            raise AnyException.default(response.text,
                                             response.status_code)

        json_response = response.json()

        try:
            tag_data = json_response['graphql']['shortcode_media'][
                'edge_media_to_tagged_user']['edges']
        except KeyError:
            return []

        tagged_users = []

        for tag in tag_data:
            x_pos = tag['node']['x']
            y_pos = tag['node']['y']
            user = tag['node']['user']
            # TODO: add Model and add Data to it instead of Dict
            tagged_user = dict()
            tagged_user['x_pos'] = x_pos
            tagged_user['y_pos'] = y_pos
            tagged_user['user'] = user

            tagged_users.append(tagged_user)

        return tagged_users

    def __is_logged_in__(self, session):
        """
        :param session: session dict
        :return: bool
        """
        if session is None or 'sessionid' not in session.keys():
            return False

        session_id = session['sessionid']
        csrf_token = session['csrftoken']

        headers = {
            'cookie': f"ig_cb=1; csrftoken={csrf_token}; sessionid={session_id};",
            'referer': InstagramEndPoints().BASE_URL + '/',
            'x-csrftoken': csrf_token,
            'X-CSRFToken': csrf_token,
            'user-agent': self.user_agent,
        }

        time.sleep(self.sleep_between_requests)
        response = self.__req.get(InstagramEndPoints().BASE_URL, headers=headers)

        if not response.status_code == HTTP_OK:
            return False

        cookies = response.cookies.get_dict()

        if cookies is None or not 'ds_user_id' in cookies.keys():
            return False

        return True

    def __login__(self, force=False, two_step_verificator=None):
        """support_two_step_verification true works only in cli mode - just run __login__ in cli mode - save cookie to file and use in any mode
        :param force: true will refresh the session
        :param two_step_verificator: true will need to do verification when an account goes wrong
        :return: headers dict
        """
        if self.session_username is None or self.session_password is None:
            raise AuthException("User credentials not provided")

        if two_step_verificator:
            two_step_verificator = TwoStepConsoleVerification()

        session = json.loads(
            instance_cache.get_saved_cookies()) if instance_cache.get_saved_cookies() != None else None

        if force or not self.__is_logged_in__(session):
            time.sleep(self.sleep_between_requests)
            response = self.__req.get(InstagramEndPoints().BASE_URL)
            if not response.status_code == HTTP_OK:
                raise AnyException.default(response.text,
                                                 response.status_code)

            match = re.findall(r'"csrf_token":"(.*?)"', response.text)

            if len(match) > 0:
                csrfToken = match[0]

            cookies = response.cookies.get_dict()

            # cookies['mid'] doesnt work at the moment so fetch it with function
            mid = self.__get_mid__()

            headers = {
                'cookie': f"ig_cb=1; csrftoken={csrfToken}; mid={mid};",
                'referer': InstagramEndPoints().BASE_URL + '/',
                'x-csrftoken': csrfToken,
                'X-CSRFToken': csrfToken,
                'user-agent': self.user_agent,
            }
            payload = {'username': self.session_username,
                       'password': self.session_password}
            response = self.__req.post(InstagramEndPoints().LOGIN_URL, data=payload,
                                       headers=headers)

            if not response.status_code == HTTP_OK:
                if (
                        response.status_code == HTTP_BAD_REQUEST
                        and response.text is not None
                        and response.json()['message'] == 'checkpoint_required'
                        and two_step_verificator is not None):
                    response = self.__verify_two_step__(response, cookies,
                                                      two_step_verificator)
                    print('checkpoint required')

                elif response.status_code is not None and response.text is not None:
                    raise AuthException(
                        f'Response code is {response.status_code}. Body: {response.text} Something went wrong. Please report issue.',
                        response.status_code)
                else:
                    raise AuthException(
                        'Something went wrong. Please report issue.',
                        response.status_code)

            if not response.json()['authenticated']:
                raise AuthException('User credentials are wrong.')

            cookies = response.cookies.get_dict()

            cookies['mid'] = mid
            instance_cache.set_saved_cookies(json.dumps(cookies, separators=(',', ':')))

            self.user_session = cookies

        else:
            self.user_session = session

        return self.__generate_headers__(self.user_session)

    def __verify_two_step__(self, response, cookies, two_step_verificator):
        """
        :param response: Response object returned by Request
        :param cookies: user cookies
        :param two_step_verificator: two_step_verification instance
        :return: Response
        """
        new_cookies = response.cookies.get_dict()
        cookies = {**cookies, **new_cookies}

        cookie_string = ''
        for key in cookies.keys():
            cookie_string += f'{key}={cookies[key]};'

        headers = {
            'cookie': cookie_string,
            'referer': InstagramEndPoints().LOGIN_URL,
            'x-csrftoken': cookies['csrftoken'],
            'user-agent': self.user_agent,
        }

        url = InstagramEndPoints().BASE_URL + response.json()['checkpoint_url']

        time.sleep(self.sleep_between_requests)
        response = self.__req.get(url, headers=headers)
        data = self.__extract_shared_data_from_body__(response.text)

        if data is not None:
            try:
                choices = \
                    data['entry_data']['Challenge'][0]['extraData']['content'][
                        3][
                        'fields'][0]['values']
            except KeyError:
                choices = dict()
                try:
                    fields = data['entry_data']['Challenge'][0]['fields']
                    try:
                        choices.update({'label': f"Email: {fields['email']}",
                                        'value': 1})
                    except KeyError:
                        pass
                    try:
                        choices.update(
                            {'label': f"Phone: {fields['phone_number']}",
                             'value': 0})
                    except KeyError:
                        pass

                except KeyError:
                    pass

            if len(choices) > 0:
                selected_choice = two_step_verificator.get_verification_type(
                    choices)
                response = self.__req.post(url,
                                           data={'choice': selected_choice},
                                           headers=headers)

        if len(re.findall('name="security_code"', response.text)) <= 0:
            raise AuthException(
                'Something went wrong when try '
                'two step verification. Please report issue.',
                response.status_code)

        security_code = two_step_verificator.get_security_code()

        post_data = {
            'csrfmiddlewaretoken': cookies['csrftoken'],
            'verify': 'Verify Account',
            'security_code': security_code,
        }
        response = self.__req.post(url, data=post_data, headers=headers)
        if not response.status_code == HTTP_OK \
                or 'Please check the code we sent you and try again' in response.text:
            raise AuthException(
                'Something went wrong when try two step'
                ' verification and enter security code. Please report issue.',
                response.status_code)

        return response

    def __add_comment__(self, media_id, text, replied_to_comment_id=None):
        """
        :param media_id: media id
        :param text:  the content you want to post
        :param replied_to_comment_id: the id of the comment you want to reply
        :return: Comment
        """
        media_id = media_id.identifier if isinstance(media_id, Media) else media_id

        replied_to_comment_id = replied_to_comment_id._data['id'] if isinstance(replied_to_comment_id, Comment) else replied_to_comment_id

        body = {'comment_text': text,
                'replied_to_comment_id': replied_to_comment_id
                if replied_to_comment_id is not None else ''}

        response = self.__req.post(InstagramEndPoints().get_add_comment_url(media_id),
                                   data=body, headers=self.__generate_headers__(
                self.user_session))

        if not HTTP_OK == response.status_code:
            raise AnyException.default(response.text,
                                             response.status_code)

        json_response = response.json()

        if json_response['status'] != 'ok':
            status = json_response['status']
            raise AnyException(
                f'Response status is {status}. '
                f'Body: {response.text} Something went wrong.'
                f' Please report issue.',
                response.status_code)

        return Comment(json_response)

    def __delete_comment__(self, media_id, comment_id):
        """
        :param media_id: media id
        :param comment_id: the id of the comment you want to delete
        """
        media_id = media_id.identifier if isinstance(media_id,
                                                     Media) else media_id

        comment_id = comment_id._data['id'] if isinstance(comment_id,
                                                          Comment) else comment_id

        response = self.__req.post(
            InstagramEndPoints().get_delete_comment_url(media_id, comment_id),
            headers=self.__generate_headers__(self.user_session))

        if not HTTP_OK == response.status_code:
            raise AnyException.default(response.text,
                                             response.status_code)

        json_response = response.json()

        if json_response['status'] != 'ok':
            status = json_response['status']
            raise AnyException(
                f'Response status is {status}. '
                f'Body: {response.text} Something went wrong.'
                f' Please report issue.',
                response.status_code)

    def __like__(self, media_id):
        """
        :param media_id: media id
        """
        media_id = media_id.identifier if isinstance(media_id,
                                                     Media) else media_id
        response = self.__req.post(InstagramEndPoints().get_like_url(media_id),
                                   headers=self.__generate_headers__(
                                       self.user_session))

        if not HTTP_OK == response.status_code:
            raise AnyException.default(response.text,
                                             response.status_code)

        json_response = response.json()

        if json_response['status'] != 'ok':
            status = json_response['status']
            raise AnyException(
                f'Response status is {status}. '
                f'Body: {response.text} Something went wrong.'
                f' Please report issue.',
                response.status_code)

    def __unlike__(self, media_id):
        """
        :param media_id: media id
        """
        media_id = media_id.identifier if isinstance(media_id,
                                                     Media) else media_id
        response = self.__req.post(InstagramEndPoints().get_unlike_url(media_id),
                                   headers=self.__generate_headers__(
                                       self.user_session))

        if not HTTP_OK == response.status_code:
            raise AnyException.default(response.text,
                                             response.status_code)

        json_response = response.json()

        if json_response['status'] != 'ok':
            status = json_response['status']
            raise AnyException(
                f'Response status is {status}. '
                f'Body: {response.text} Something went wrong.'
                f' Please report issue.',
                response.status_code)

    def __follow__(self, user_id):
        """
        :param user_id: user id
        :return: bool
        """
        if self.__is_logged_in__(self.user_session):
            url = InstagramEndPoints().get_follow_url(user_id)

            try:
                follow = self.__req.post(url,
                                         headers=self.__generate_headers__(
                                             self.user_session))
                if follow.status_code == HTTP_OK:
                    return True
            except:
                raise AnyException("Except on follow!")
        return False

    def __unfollow__(self, user_id):
        """
        :param user_id: user id
        :return: bool
        """
        if self.__is_logged_in__(self.user_session):
            url_unfollow = InstagramEndPoints().get_unfollow_url(user_id)
            try:
                unfollow = self.__req.post(url_unfollow)
                if unfollow.status_code == HTTP_OK:
                    return unfollow
            except:
                raise AnyException("Exept on unfollow!")
        return False


from hackingtools.core import Logger, Utils, Config
import hackingtools as ht
import os

import shutil
from zipfile import ZipFile

config = Config.getConfig(parentKey='modules', key='ht_unzip')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():

	def __init__(self):
		Utils.emptyDirectory(output_dir)
		self._main_gui_func_ = 'extractFilePassword'

	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_unzip'), debug_module=True)

	def extractFilePassword(self, zipPathName, password='', posible_combinations=1, output_dir_new=None):
		#ZipFile only works with 7z with ZypCrypto encryption for setting the password
		try:
			Logger.printMessage(message="extractFilePassword", description='ZIP - {pwd} - {msg_posible_comb}: {com}'.format(pwd=password, msg_posible_comb=config['posible_combinations'], com=posible_combinations), debug_module=True)
			with ZipFile(zipPathName) as zf:
				zf.extractall(output_dir if not output_dir_new else output_dir_new, pwd=str.encode(password))
			return password
		except Exception as e:
			Logger.printMessage(message="extractFilePassword", description='{e} - {p}'.format(e=str(e), p=password), is_error=True)
			return None

	def extractFile(self, zipPathName, password=None):
		#ZipFile only works with 7z with ZypCrypto encryption for setting the password
		try:
			with ZipFile(zipPathName) as zf:
				return zf.extractall(password) if password else zf.extractall(os.path.split(zipPathName)[0])
		except Exception as e:
			Logger.printMessage(message="extractFile", description=str(e), is_error=True)
			return None
	
	def zipDirectory(self, new_folder_name):
		# Creates a Zip File from a directory
		return shutil.make_archive(new_folder_name, 'zip', new_folder_name)


from hackingtools.core import Logger, Config, Utils
import hackingtools as ht

import nmap
import os

config = Config.getConfig(parentKey='modules', key='ht_nmap')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():
	
	def __init__(self):
		self._main_gui_func_ = 'getCVEsFromHost'

	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_nmap'), debug_module=True)

	def getConnectedDevices(self, ip):
		Logger.printMessage(message='{methodName}'.format(methodName='getConnectedDevices'), description='{param}'.format(param=ip), debug_module=True)
		nm = nmap.PortScanner()
		results = nm.scan(hosts=ip, arguments='-n -sP -PE -PA21,23,80,3389')
		if 'scan' in results: 
			hosts = {}
			for host in results['scan']:
				if results['scan'][host]['status']['state'] == 'up':
					mac_address = None

					if 'mac' in results['scan'][host]['addresses']:
						mac_address = results['scan'][host]['addresses']['mac']

					hosts[host] = '' if not mac_address else results['scan'][host]['vendor'][ mac_address ]
					
			return hosts
		return []

	def executeScan(self, ip, params=''):
		Logger.printMessage(message='{methodName}'.format(methodName='executeScan'), description='{param}'.format(param=ip), debug_module=True)
		nm = nmap.PortScanner()
		try:
			return dict( nm.scan(hosts=ip, arguments=params) )
		except:
			return {}

	def getCVEsFromHost(self, ip):
		Logger.printMessage(message='{methodName}'.format(methodName='getConnectedDevices'), description='{param}'.format(param=ip), debug_module=True)
		nm = nmap.PortScanner()
		results = nm.scan(hosts=ip, arguments='-Pn --script vuln')
		protocols = ('tcp', 'udp')
		discard_vuln_by_description = ('ERROR:', 'Couldn\'t', '\n  /jmx-console/: Authentication was not required\n')
		res = {}
		for p in protocols:
			if 'scan' in results and ip in results['scan']:
				if p in results['scan'][ip]:
					for port in results['scan'][ip][p]:
						res[port] = []
						if 'script' in results['scan'][ip][p][port]:
							for vuln in results['scan'][ip][p][port]['script']:
								discard = False
								for dis in discard_vuln_by_description:
									if results['scan'][ip][p][port]['script'][vuln].startswith(dis):
										discard = True
								if not discard:
									if not results['scan'][ip][p][port]['script'][vuln] == '\n':
										if 'CVE:' in results['scan'][ip][p][port]['script'][vuln]:
											res[port].append( { vuln : results['scan'][ip][p][port]['script'][vuln].split('CVE:')[1].split('\n')[0].split(' ')[0] } )
										else:
											res[port].append( { vuln : results['scan'][ip][p][port]['script'][vuln] } )
						if len(res[port]) == 0:
							del res[port]
		
		return res

	def getDevicePorts(self, ip, tcp=True, udp=False):
		Logger.printMessage(message='{methodName}'.format(methodName='getDevicePorts'), description='{param} - TCP {tcp} - UDP {udp}'.format(param=ip, tcp=tcp, udp=udp), debug_module=True)
		nm = nmap.PortScanner()
		results = nm.scan(ip)
		try:
			if tcp and not udp:
				return Utils.getValidDictNoEmptyKeys(results["scan"][ip]["tcp"])
			if udp and not tcp:
				return Utils.getValidDictNoEmptyKeys(results["scan"][ip]["udp"])
			if tcp and udp:
				return Utils.getValidDictNoEmptyKeys([results["scan"][ip]["tcp"],results["scan"][ip]["udp"]])
			return Utils.getValidDictNoEmptyKeys(results["scan"][ip]["tcp"])
		except:
			return []

	def hasDevicePortOpened(self, ip, port):
		Logger.printMessage(message='{methodName}'.format(methodName='hasDevicePortOpened'), description='{param}:{param2}'.format(param=ip, param2=port), debug_module=True)
		nm = nmap.PortScanner()
		results = nm.scan(ip)
		exists = False
		try:
			for host in results.all_hosts():
				if exists:
					break
				if not exists:
					exists = results[host].has_tcp(port)
				if not exists:
					exists = results[host].has_udp(port)
		except:
			Logger.printMessage(message="isBadFile", description=results, is_error=True)
			raise
		return exists



from . import Config, Logger, Utils
from .Objects import Transaction
if Utils.amIdjango(__name__):
    from core.library import hackingtools as ht
else:
    import hackingtools as ht
from django.urls import resolve as __resolve
import requests as __requests
import json as __json
import random as __random
import time as __time

import binascii
from Crypto import Random
from Crypto.PublicKey import RSA

# Nodes Pool Treatment

__nodes_pool__ = Config.getConfig(parentKey='core', key='Pool', subkey='known_nodes')

global __CHECKED_NODES__
__CHECKED_NODES__ = False
__MY_NODE_ID__ = Utils.randomText(length=32, alphabet='mixalpha-numeric-symbol14')

global __BLOCKCHAIN_MASTERNODE__
__BLOCKCHAIN_MASTERNODE__ = None

global __CURRENT_BLOCKCHAIN__
__CURRENT_BLOCKCHAIN__ = None

global __PENDING_TRANSACTIONS__
__PENDING_TRANSACTIONS__ = None

def newWallet():
	random_gen = Random.new().read
	private_key = RSA.generate(1024, random_gen)
	public_key = private_key.publickey()
	response = {
		'private_key': binascii.hexlify(private_key.exportKey(format='DER')).decode('ascii'),
		'public_key': binascii.hexlify(public_key.exportKey(format='DER')).decode('ascii')
	}
	return response['private_key'], response['public_key']

def __generateWalletTransaction__(value, sent_to_wallet=''):
	transaction = Transaction(__WALLET_PUBLIC_KEY__, __WALLET_PRIVATE_KEY__, sent_to_wallet, value)
	response = {'transaction': transaction.to_dict(), 'signature': transaction.sign_transaction()}
	return response['transaction'], response['signature']

def sendTransaction(value, sent_to_wallet=''):
    global __BLOCKCHAIN_MASTERNODE__
    global __CURRENT_BLOCKCHAIN__
    if __BLOCKCHAIN_MASTERNODE__:
        if not __CURRENT_BLOCKCHAIN__:
            getBlockchain()
        transaction, signature = __generateWalletTransaction__(value, sent_to_wallet)
        url = 'http://{url}/transactions/new'.format(url=__BLOCKCHAIN_MASTERNODE__)
        data = {
            'sender_address': transaction['sender_address'], 
            'recipient_address': transaction['recipient_address'], 
            'amount': transaction['value'], 
            'signature': signature
        }
        response = __requests.post( url, data )
        return __json.loads( response.text )
    else:
        Logger.printMessage( 'There is no Master Node for the Wallet configured yet', is_error=True )

def getBlockchain():
    global __BLOCKCHAIN_MASTERNODE__
    global __CURRENT_BLOCKCHAIN__
    if __BLOCKCHAIN_MASTERNODE__:
        try:
            url = 'http://{url}/chain'.format(url=__BLOCKCHAIN_MASTERNODE__)
            response = __requests.get(url)
            if response.status_code == 200:
                __CURRENT_BLOCKCHAIN__ = __json.loads( response.text )
            else:
                Logger.printMessage( response.status_code, is_error=True )
        except Exception as e:
            Logger.printMessage( str(e), is_error=True )
    else:
        Logger.printMessage( 'There is no Master Node for the Wallet configured yet', is_error=True )

def getPendingTransactions():
    global __BLOCKCHAIN_MASTERNODE__
    global __PENDING_TRANSACTIONS__
    global __WALLET_PUBLIC_KEY__
    if __BLOCKCHAIN_MASTERNODE__:
        try:
            url = 'http://{url}/transactions/get'.format(url=__BLOCKCHAIN_MASTERNODE__)
            response = __requests.get(url)
            if response.status_code == 200:
                __PENDING_TRANSACTIONS__ = [ tran for tran in __json.loads( response.text )['transactions'] if not tran['sender_address'] == __WALLET_PUBLIC_KEY__ ]
            else:
                Logger.printMessage( response.status_code, is_error=True )
        except Exception as e:
            Logger.printMessage( str(e), is_error=True )
    else:
        Logger.printMessage( 'There is no Master Node for the Wallet configured yet', is_error=True )

def setMasterNode(nodeUrl):
    global __BLOCKCHAIN_MASTERNODE__
    __BLOCKCHAIN_MASTERNODE__ = nodeUrl.replace('http://', '').replace('https://', '').split('/')[0]

def minePendingTransaction():
    global __BLOCKCHAIN_MASTERNODE__
    global __WALLET_PUBLIC_KEY__
    if __BLOCKCHAIN_MASTERNODE__:
        url = 'http://{url}/mine'.format(url=__BLOCKCHAIN_MASTERNODE__)
        data = {
            'sender_address': __WALLET_PUBLIC_KEY__
        }
        response = __requests.post( url, data )
        return __json.loads( response.text )
    else:
        Logger.printMessage( 'There is no Master Node for the Wallet configured yet', is_error=True )

__WALLET_PRIVATE_KEY__, __WALLET_PUBLIC_KEY__ = newWallet()

def switchPool():
    ht.switchPool()

def addNodeToPool(node_ip):
    global __nodes_pool__
    global __CHECKED_NODES__
    if node_ip and not node_ip in __nodes_pool__:
        if Utils.amIdjango(__name__):
            Logger.printMessage(node_ip, debug_core=True)
        if not node_ip in ht.__Connections.getMyServices():
            __nodes_pool__.append(node_ip)
            Config.add_pool_node(node_ip)
            __CHECKED_NODES__ = False

def sendNow(moduleName, functionName, params={}, files={}):
    global __nodes_pool__

    url_path = 'modules/{cat}/{mod}/{func_call}/'.format(cat=ht.getModuleCategory(moduleName), mod=moduleName.replace('ht_', ''), func_call=functionName)

    params['creator'] = __MY_NODE_ID__
    params['pool_list'] = []
            
    response, _, resolver = __sendPool__(__MY_NODE_ID__, url_path, params, files)

    while 'NoneType' in response:
        response, _, resolver = __sendPool__(__MY_NODE_ID__, url_path, params, files)
        
    res = {}
    res['__nodes_pool__'] = __nodes_pool__
    res['resolved_by'] = resolver

    if isinstance(response, dict):
        res['res'] = response['data']
    else:
        res['res'] = response
    
    return res

def getPoolNodes():
    global __nodes_pool__
    __random.shuffle(__nodes_pool__)
    return __nodes_pool__

def removeNodeFromPool(node_ip):
    global __nodes_pool__
    if node_ip in __nodes_pool__:
        __nodes_pool__.remove(node_ip)
    Config.remove_pool_node(node_ip)

def checkNode(node):
    idnode = __getNodeId__(node)
    if idnode == __MY_NODE_ID__:
        Logger.printMessage('Removing node from __nodes_pool__, im this service xD', node, debug_core=True)
        removeNodeFromPool(node)
        if not node in ht.__Connections.getMyServices():
            Config.add_my_service(node) 

def __callNodesForInformAboutMyServices__():
    global __nodes_pool__ 
    for n in __nodes_pool__:
        ngrok_url = ht.__Connections.getNgrokServiceUrl()
        service_for_call = '{node_ip}/core/pool/add_pool_node/'.format(node_ip=n)
        if ngrok_url:
            try:
                if ngrok_url:
                    __requests.post(service_for_call, data={'pool_ip':ngrok_url , 'pooling':True},  headers=ht.__Connections.headers)
            except:
                pass
        else:
            for serv in ht.__Connections.getMyServices():
                try:
                    if serv:
                        __requests.post(service_for_call, data={'pool_ip':serv , 'pooling':True},  headers=ht.__Connections.headers)
                except:
                    pass

def __send__(node_request, functionName):
    creator_id = __MY_NODE_ID__
    pool_nodes = getPoolNodes()
    try:
        if ht.wantPool():
            function_api_call = __resolve(node_request.path_info).route
            pool_it = node_request.POST.get('__pool_it_{func}__'.format(func=functionName), False)
            if pool_it:
                if pool_nodes:
                    params = dict(node_request.POST)
                    if 'pool_list' in params:
                        if not params['pool_list']:
                            params['pool_list'] = []
                    if 'creator' in params:
                        if params['creator'] == __MY_NODE_ID__:
                            Logger.printMessage(message='My own call', description='Discarding...', is_warn=True)
                            return (None, None)
                    if not 'creator' in params:
                        params['creator'] = creator_id

                    if params['creator'] == creator_id: # TODO This disables repooling ------ SOLVE THIS PLEASEEE WE NEED REPOOOL
                        response, creator, _ = __sendPool__(creator=params['creator'], function_api_call=function_api_call, params=dict(params), files=node_request.FILES)
                        
                        __callNodesForInformAboutMyServices__()
                                
                        if 'creator' in params and params['creator'] == creator_id and response:
                            if isinstance(response, str):
                                return ({ 'res' : response, '__nodes_pool__' : __nodes_pool__ }, creator_id)
                            if isinstance(response, dict):
                                return ({ 'res' : response['data'], '__nodes_pool__' : __nodes_pool__ }, creator_id)
                            try:
                                return (str(response.text), False)
                            except:
                                return (str(response), False)
                        if response:
                            return (response, creator) # Repool
                    return (None, None)
                else:
                    return (None, None)
            else:
                return (None, None)
        else:
            Logger.printMessage(message='Disabled pool', description='If want to pool, change __WANT_TO_BE_IN_POOL__ to true', is_warn=True)
            return (None, None)
    except Exception as e:
        Logger.printMessage(message='ERROR', description=str(e), is_error=True)
        return (None, None)

def __sendPool__(creator, function_api_call='', params={}, files=[]):
    # We have 3 diferent nodes list:
    #   1- __nodes_pool__ : We know those nodes for any call
    #   2- pool_list : is inside params['pool_list'] and has the list of all pools that know this pool request
    #   3- nodes : Are the nodes we can call from out __nodes_pool__ and that the aren't inside pool_list
    # Finally we add all pool_list nodes that aren't inside our __nodes_pool__ to __nodes_pool__ list

    global __nodes_pool__

    nodes = [] # Nodes to send this call. Thay have to be nodes that haven't received this yet.
    pool_list = [] # The pool_list is a list for getting all the nodes that have been notified by this call.

    mine_function_call = False

    try:
        pool_list = params['pool_list']
        for service in ht.__Connections.getMyServices():
            if service in pool_list:
                mine_function_call = True
                Logger.printMessage(message=function_api_call, description='It\'s my own call', is_warn=True)
                return (None, None, None)
    except:
        pass
    
    nodes = __nodes_pool__
    if pool_list:
        nodes = list(set(__nodes_pool__) - set(pool_list))

    # Get all nodes in pool_list as known for us if we don't have any
    if not __nodes_pool__:
        __nodes_pool__ = pool_list

    # I save pool_list items i don't have yet on my pools

    __nodes_pool__ = __nodes_pool__ + list(set(pool_list) - set(__nodes_pool__))

    my_own_call = False

    if pool_list:
        for service in ht.__Connections.getMyServices():
            if service in pool_list:
                my_own_call = True
                pool_list.remove(service)

    # Remove any posible service with my public, local or lan IP
    if __nodes_pool__:
        for service in ht.__Connections.getMyServices():
            removeNodeFromPool(service)

    __random.shuffle(nodes)

    if len(nodes) > 0:
        if not mine_function_call and not my_own_call:
            for node in nodes:
                try:
                    if ht.__Connections.__serviceNotMine__(node):

                        node_call = '{node_ip}/pool/execute/'.format(node_ip=node)

                        params['pool_list'] = pool_list

                        for serv in ht.__Connections.getMyServices():
                            params['pool_list'].append(serv)
                            
                        params['is_pool'] = True

                        params['functionCall'] = function_api_call

                        if not 'creator' in params:
                            params['creator'] == __MY_NODE_ID__
                        
                        if node not in params['pool_list']:
                            print(files)
                            r = __requests.post(node_call, files=files, data=params, headers=dict(Referer=node))
                        
                            if r.status_code == 200:
                                for n in pool_list:
                                    if ht.__Connections.__serviceNotMine__(n) and not n == node:
                                        addNodeToPool(n)
                                Logger.printMessage(message='Solved by', description=(node), debug_core=True)
                                try:
                                    Logger.printMessage(__json.loads(str(r.text))['data'], debug_core=True)
                                except:
                                    Logger.printMessage(r, debug_core=True)
                                return (__json.loads(str(r.text))['data'], params['creator'], node) # (Data Content, Creator Call, Resolved By Node)
                except Exception as e:
                    Logger.printMessage(message='ERROR', description=str(e), is_warn=True)
        else:
            Logger.printMessage(message='ERROR', description='Returned to me my own function called into the pool', debug_core=True)
    else:
        Logger.printMessage(message='ERROR', description='There is nobody on the pool list', debug_core=True)

    return (None, None, None)

def __getNodeId__(node, thread=False):
    threaded = thread

    if not thread:
        threaded = True

    while threaded:

        url = '{url}/core/pool/getNodeId/'.format(url=node)

        if thread:
            Logger.printMessage('Call to the node for not been idle in 3 minutes', url, color=Logger.__Fore.YELLOW)
            __time.sleep(180)

        try:
            r = __requests.post(url, headers=ht.__Connections.__headers__)
            if r.status_code == 200:
                return r.json()['data']
        except:
            Logger.printMessage('Error connecting to server, removing node from pool', url, is_error=True)
            removeNodeFromPool(node)

        if not thread:
            threaded = False

    return None

def __checkPoolNodes__(thread=False):
    global __CHECKED_NODES__
    changes = False
    if not __CHECKED_NODES__:
        for node in getPoolNodes():
            idnode = __getNodeId__(node, thread)
            if idnode == __MY_NODE_ID__:
                checkNode(node)
                changes = True
            __CHECKED_NODES__ = True
        if not getPoolNodes():
            Config.__djangoSwitchPoolItButtons__(False)
            return False
    
    if getPoolNodes() and changes:
        Config.__djangoSwitchPoolItButtons__(True)
        return True
    return False

# t = ht.Utils.worker(functionCall=__checkPoolNodes__, args=(True,))


from . import Config, Logger, Utils
import requests as __requests
import json as __json
import shutil as __shutil
import os as __os

if Utils.amIdjango(__name__):
    from core.library import hackingtools as ht
else:
    import hackingtools as ht

config = Config.getConfig(parentKey='core', key='Repositories')

__path = __os.path.abspath(__os.path.split(__os.path.dirname(__file__))[0])

__servers__ = config['servers']

def installModule(server, moduleName):
    req = __requests.post('http://{ip}/category/{m}'.format(ip=server, m=moduleName))

    if req.json()['status'] == 'OK':
        category = req.json()['data']

        # Installing main files for the module

        req = __requests.get('http://{ip}/module/download/files/{m}'.format(ip=server, m=moduleName), stream=True, data={'module_name' : moduleName}, headers={'Accept-Encoding' : '*/*', 'Content-Type' : 'application/zip'})

        category_folder = __os.path.join(__path, 'modules', category)
        if not __os.path.isdir(category_folder):
            __os.mkdir(category_folder)

        new_file = __os.path.join(category_folder, '{m}.zip'.format(m=moduleName))
        open(new_file, 'wb').write(req.content)

        unzipper = ht.getModule('ht_unzip')
        unzipper.extractFile(new_file)

        __os.remove(new_file)

        # Installing the json config
        req = __requests.get('http://{ip}/module/download/config/{m}'.format(ip=server, m=moduleName), stream=True, data={'module_name' : moduleName}, headers={'Accept-Encoding' : '*/*', 'Content-Type' : 'application/json'})
        
        config_django_module_path = __os.path.abspath(__os.path.join(__os.path.dirname(__file__), 'config_modules_django', category))
        if not __os.path.isdir(config_django_module_path):
            __os.mkdir(config_django_module_path)
        
        open(__os.path.join(config_django_module_path, 'ht_{m}.json'.format(m=moduleName.replace('ht_',''))), 'wb').write(req.content)

def updateModule(server, moduleName):
    __removeModule__(server, moduleName)
    installModule(server, moduleName)

def __removeModule__(server, moduleName):
    moduleName = moduleName.replace('ht_', '')
    req = __requests.post('http://{ip}/category/{m}'.format(ip=server, m=moduleName))

    if req.json()['status'] == 'OK':
        category = req.json()['data']

        category_folder = __os.path.join(__path, 'modules', category)
        if __os.path.isdir(category_folder):
            module_folder = __os.path.join(category_folder, moduleName)
            if __os.path.isdir(module_folder):
                __shutil.rmtree(module_folder)

def uploadModule(server, category, moduleName):
    module_folder = __os.path.join(__path, 'modules', category, moduleName.replace('ht_', ''))
    unzipper = ht.getModule('ht_unzip')

    # Zip the module directory
    zipped_file = unzipper.zipDirectory(module_folder)

    # Create Uploads directory
    uploadsFolder = __os.path.join(__os.path.join(__os.path.split(zipped_file)[0]), '__uploads__')
    if not __os.path.isdir(uploadsFolder):
        __os.mkdir(uploadsFolder)

    # Create Module directory inside Uploads
    moduleUploadsFolder = __os.path.join(uploadsFolder, moduleName.replace('ht_', ''))
    if not __os.path.isdir(moduleUploadsFolder):
        __os.mkdir(moduleUploadsFolder)
    
    # Move zipped module to uploads dir
    new_file_zipped = __os.path.join(moduleUploadsFolder, __os.path.split(zipped_file)[-1])
    try:
        __os.rename(zipped_file, new_file_zipped)
    except:
        try:
            if __os.path.isfile(new_file_zipped):
                __os.remove(new_file_zipped)
            __os.rename(zipped_file, new_file_zipped)
        except:
            pass

    # Copy the JSON config for the module into the uploads dir
    moduleConfigFile = __os.path.join(__path, 'core', 'config_modules_django', category, 'ht_{f}.json'.format(f=moduleName.replace('ht_', '')))
    newModuleConfigFile = __os.path.join(moduleUploadsFolder, 'ht_{f}.json'.format(f=moduleName.replace('ht_', '')))
    if __os.path.isfile(moduleConfigFile):
        if __os.path.isfile(newModuleConfigFile):
            __os.remove(newModuleConfigFile)
        __shutil.copyfile(moduleConfigFile, newModuleConfigFile)

    # Copy the Django view for the module into the uploads dir
    hackingtoolsDjangoCoreFolder = __os.path.split(__os.path.split(__path)[0])[0]
    moduleViewsFile = __os.path.join(hackingtoolsDjangoCoreFolder, 'views_modules', category, 'views_ht_{f}.py'.format(f=moduleName.replace('ht_', '')))
    newModuleViewsFile = __os.path.join(moduleUploadsFolder, 'views_ht_{f}.py'.format(f=moduleName.replace('ht_', '')))
    if __os.path.isfile(moduleViewsFile):
        if __os.path.isfile(newModuleViewsFile):
            __os.remove(newModuleViewsFile)
        __shutil.copyfile(moduleViewsFile, newModuleViewsFile)

    # Copy the requirements file for the module into the uploads dir
    moduleRequirementsFile = __os.path.join(__path, 'modules', '__requirements__', category, 'ht_{f}.txt'.format(f=moduleName.replace('ht_', '')))
    newModuleRequirementsFile = __os.path.join(moduleUploadsFolder, 'ht_{f}.txt'.format(f=moduleName.replace('ht_', '')))
    if __os.path.isfile(moduleRequirementsFile):
        if __os.path.isfile(newModuleRequirementsFile):
            __os.remove(newModuleRequirementsFile)
        __shutil.copyfile(moduleRequirementsFile, newModuleRequirementsFile)

    # Zip all together
    full_zipped_file = unzipper.zipDirectory(moduleUploadsFolder)

    files = {
        moduleName.replace('ht_', ''): open(full_zipped_file, 'rb'),
    }
    req = __requests.post('http://{ip}/new/module/upload/{c}'.format(ip=server, c=category), files=files)
    if req.json()['status'] == 'OK':
        Logger.printMessage(req.json()['data'], debug_core=True)
    else:
        Logger.printMessage(req.json()['data'], is_error=True)

def clearUploadsTemp():
    for category in ht.getCategories():
        uploadFirCategory = __os.path.join(__path, 'modules', category, '__uploads__')
        if __os.path.isdir(uploadFirCategory):
            __shutil.rmtree(uploadFirCategory)

def addServer(server):
    if not server in __servers__:
        __servers__.append(server)

def getOnlineServers():
    alive_servers = []
    for serv in __servers__:
        try:
            r = __requests.post('http://{ip}/'.format(ip=serv))
            if r.ok and r.json()['status'] == 'OK':
                alive_servers.append(serv)
        except Exception as e:
            Logger.printMessage(str(e), is_error=True)
    return alive_servers
    
def getChanges(server):
    try:
        r = __requests.post('http://{ip}/changes/'.format(ip=server))
        if r.ok and r.json()['status'] == 'OK':
            return r.json()['data']
        return __json.dumps(r.json())
    except Exception as e:
        return __json.dumps({'status':  'FAIL', 'data': str(e)})

def getCategories(server):
    try:
        r = __requests.post('http://{ip}/categories/'.format(ip=server))
        if r.ok and r.json()['status'] == 'OK':
            return r.json()['data']
        return __json.dumps(r.json())
    except Exception as e:
        return __json.dumps({'status':  'FAIL', 'data': str(e)})

def getModules(server):
    try:
        r = __requests.post('http://{ip}/modules/'.format(ip=server))
        if r.ok and r.json()['status'] == 'OK':
            return r.json()['data']
        return __json.dumps(r.json())
    except Exception as e:
        return __json.dumps({'status':  'FAIL', 'data': str(e)})

def getModulesNamesByCategory(server, category):
    try:
        r = __requests.post('http://{ip}/category/{cat}'.format(ip=server, cat=category))
        if r.ok and r.json()['status'] == 'OK':
            return r.json()['data']
        return __json.dumps(r.json())
    except Exception as e:
        return __json.dumps({'status':  'FAIL', 'data': str(e)})

def downloadModuleFull(server, moduleName):
    try:
        r = __requests.post('http://{ip}/download/{mod}'.format(ip=server, mod=moduleName))
        return __json.dumps(r.json())
    except Exception as e:
        return __json.dumps({'status':  'FAIL', 'data': str(e)})

def downloadModuleFiles(server, moduleName):
    try:
        r = __requests.post('http://{ip}/download/files/{mod}'.format(ip=server, mod=moduleName))
        return __json.dumps(r.json())
    except Exception as e:
        return __json.dumps({'status':  'FAIL', 'data': str(e)})

def downloadModuleConf(server, moduleName):
    try:
        r = __requests.post('http://{ip}/download/config/{mod}'.format(ip=server, mod=moduleName))
        return __json.dumps(r.json())
    except Exception as e:
        return __json.dumps({'status':  'FAIL', 'data': str(e)})

def downloadModuleDjangoView(server, moduleName):
    try:
        r = __requests.post('http://{ip}/download/views/{mod}'.format(ip=server, mod=moduleName))
        return __json.dumps(r.json())
    except Exception as e:
        return __json.dumps({'status':  'FAIL', 'data': str(e)})

def newModuleUpload(server, category, moduleName):
    try:
        r = __requests.post('http://{ip}/new/module/upload/{cat}/{mod}/'.format(ip=server, cat=category, mod=moduleName))
        return __json.dumps(r.json())
    except Exception as e:
        return __json.dumps({'status':  'FAIL', 'data': str(e)})

#getOnlineServers()



from core.library.hackingtools.core import Utils
from core.library import hackingtools as ht
import os, inspect

__default_view_init__ = """from django.http import HttpResponse, JsonResponse
import os
from requests import Response

from core import views
from core.views import ht, config, renderMainPanel, saveFileOutput, Logger, sendPool, returnAsModal

# Create your views here.
"""

__default_view_function_init__ = "\n# Automatic view function for {funcName}\ndef {funcName}(request):\n"

__lastly_added_func__ = ["help"]

def createTemplateFunctionForModule(moduleName, category, functionName):
    try:
        __createViewFileForModule__(moduleName, category)
        fileView =  __getModuleViewFilePath__(moduleName, category)

        if not functionName in  __lastly_added_func__:
            with open(fileView, 'a+') as f:
                f.write( __default_view_function_init__.format(funcName=functionName, mod=moduleName.replace('ht_', '')))

                f.write( __getViewTemplateByFunctionParams__(moduleName, functionName, category))

            __lastly_added_func__.append(functionName)
            ht.Logger.printMessage(message='Added a view for the function', description=functionName, debug_core=True)

        ht.DjangoFunctions.__createModuleFunctionView__(category, moduleName, functionName)

    except Exception as e:
        ht.Logger.printMessage(message=str(e), is_error=True)
        ht.Logger.printMessage(message='createTemplateFunctionForModule', description='Something wen\'t wrong creating template function modal view for {m}'.format(m=moduleName), is_error=True)

def loadModuleFunctionsToView(moduleName, category):
    try:
        #moduleFunctions = ht.getFunctionsNamesFromModule(moduleName).remove('help')
         __createViewFileForModule__(moduleName, category)
    except:
        ht.Logger.printMessage(message='loadModuleFunctionsToView', description='Something wen\'t wrong creating views file for {m}'.format(m=moduleName), is_error=True)

def restartDjangoServer():
    wsgi_file = os.path.join(os.path.split(os.path.split(os.path.split(os.path.split(os.path.abspath(os.path.dirname(__file__)))[0])[0])[0])[0], 'hackingtoolsgui', 'wsgi.py')
    if os.path.isfile(wsgi_file):
        ht.Logger.printMessage('Restarting the server!', color=ht.Logger.__Fore.YELLOW)
        with open(wsgi_file, 'a+')as f:
            f.write('\n')
        lines = open(wsgi_file, 'r').readlines()
        del lines[-1]
        open(wsgi_file, 'w').writelines(lines)

def removeModuleView(moduleName, category):
    moduleViewPath = os.path.join(os.path.split(os.path.split(os.path.split(os.path.abspath(os.path.dirname(__file__)))[0])[0])[0], 'views_modules', category, 'views_ht_{m}.py'.format(m=moduleName.replace('ht_', '')))
    if os.path.isfile(moduleViewPath):
        os.remove(moduleViewPath)
    moduleConfPath = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'config_modules_django', category, 'ht_{m}.py'.format(m=moduleName.replace('ht_', '')))
    if os.path.isfile(moduleConfPath):
        os.remove(moduleConfPath)

def  __getModuleViewFilePath__(moduleName, category):
    moduleViewFile = 'views_ht_{n}.py'.format(n=moduleName.replace('ht_', ''))
    categoryDir =  __getModuleViewCategoryDir__(moduleName, category)
    filePath = os.path.join(categoryDir, moduleViewFile)
    return filePath

def  __getModuleViewCategoryDir__(moduleName, category):
    actualDir = os.path.join('core', 'views_modules')
    categoryDir = os.path.join(actualDir, category)
    return categoryDir

def  __getViewTemplateByFunctionParams__(moduleName, functionName, category, params=[]):
    params = Utils.getFunctionsParams(category=category, moduleName=moduleName, functionName=functionName)
    template = '\t# Init of the view {f}\n\ttry:'.format(f=functionName)
    functionParamsForCallInStr = ""

    # Add pool condition:
    template = '{temp}\n\t\t# Pool call\n'.format(temp=template)
    template = '{temp}\t\tresponse, repool = sendPool(request, \'{f}\')\n'.format(temp=template, f=functionName)
    template = '{temp}\t\tif response or repool:\n'.format(temp=template)
    template = '{temp}\t\t\tif repool:\n'.format(temp=template)
    template = '{temp}\t\t\t\treturn HttpResponse(response)\n'.format(temp=template)
    temp = 'return JsonResponse({ "data" : str(response) })'
    template = '{temp}\t\t\t{t}\n'.format(temp=template, t=temp)
    template = '{temp}\t\telse:'.format(temp=template)

    if params:

        if 'params' in params:
            for p in params['params']:
                
                if 'file' in str(p).lower() or 'path' in str(p).lower():
                    template = '{temp}\n\t\t\ttry:\n\t\t\t\t# Save file {p}\n'.format(temp=template, p=p)
                    template = '{temp}\t\t\t\tfilename_{p}, location_{p}, {p} = saveFileOutput(request.FILES[\'{p}\'], \'{mod_no_extension}\', \'{category}\')'.format(temp=template, p=p, mod_no_extension=moduleName.replace('ht_', ''), category=ht.getModuleCategory(moduleName))
                    template = '{temp}\n\t\t\texcept Exception as e:\n\t\t\t\t# If not param {p}'.format(temp=template, p=p)
                    temp = 'return JsonResponse({ "data" : str(e) })\n\t\t\t\treturn renderMainPanel(request=request, popup_text=str(e))'
                    template = '{temp}\n\t\t\t\tif request.POST.get(\'is_async_{f}\', False):\n\t\t\t\t\t{t}'.format(temp=template, f=functionName, t=temp)
                else:
                    template = '{temp}\n\t\t\t# Parameter {p}\n'.format(temp=template, p=p)
                    template = '{temp}\t\t\t{p} = request.POST.get(\'{p}\')\n'.format(temp=template, p=p)
                functionParamsForCallInStr = '{f} {p}={p},'.format(f=functionParamsForCallInStr, p=p)

        if 'defaults' in params:
            for p in params['defaults']:

                if 'file' in str(p).lower() or 'path' in str(p).lower():
                    template = '{temp}\n\t\t\ttry:\n\t\t\t\t# Save file {p} (Optional)\n'.format(temp=template, p=p)
                    template = '{temp}\t\t\t\tfilename_{p}, location_{p}, {p} = saveFileOutput(request.FILES[\'{p}\'], \'{mod_no_extension}\', \'{category}\')'.format(temp=template, p=p, mod_no_extension=moduleName.replace('ht_', ''), category=ht.getModuleCategory(moduleName))
                    template = '{temp}\n\t\t\texcept Exception as e:\n\t\t\t\t# If not param {p}'.format(temp=template, p=p)
                    temp = 'return JsonResponse({ "data" : str(e) })\n\t\t\t\treturn renderMainPanel(request=request, popup_text=str(e))\n'
                    template = '{temp}\n\t\t\t\tif request.POST.get(\'is_async_{f}\', False):\n\t\t\t\t\t{t}'.format(temp=template, f=functionName, t=temp)
                else:
                    val = params['defaults'][p]
                    template = '{temp}\n\t\t\t# Parameter {p} (Optional - Default {v})\n'.format(temp=template, v=val, p=p)
                    
                    if isinstance(val, str):
                        template = '{temp}\t\t\t{p} = str(request.POST.get(\'{p}\', \'{v}\'))\n'.format(temp=template, p=p, v=val)
                    elif isinstance(val, bool) or val in ('True', 'False'):
                            template = '{temp}\t\t\t{p} = request.POST.get(\'{p}\', {v})\n'.format(temp=template, p=p, v=val)
                    elif isinstance(val, int):
                            template = '{temp}\t\t\t{p} = int(request.POST.get(\'{p}\', {v}))\n'.format(temp=template, p=p, v=val)
                    elif isinstance(val, float):
                            template = '{temp}\t\t\t{p} = float(request.POST.get(\'{p}\', {v}))\n'.format(temp=template, p=p, v=val)
                    elif val == 'None' or not val:
                            template = '{temp}\t\t\t{p} = request.POST.get(\'{p}\', {v})\n'.format(temp=template, p=p, v=val)
                    else:
                        template = '{temp}\t\t\t{p} = request.POST.get(\'{p}\', \'{v}\')\n'.format(temp=template, p=p, v=val)

                    if val != 0 and not val:
                        template = '{temp}\t\t\tif not {p}:\n'.format(temp=template, p=p)
                        template = '{temp}\t\t\t\t{p} = None\n'.format(temp=template, p=p)

                if not p in functionParamsForCallInStr:
                    functionParamsForCallInStr = '{f} {p}={p},'.format(f=functionParamsForCallInStr, p=p)

    functionParamsForCallInStr = functionParamsForCallInStr[:-1]

    if Utils.doesFunctionContainsExplicitReturn(Utils.getFunctionFullCall(moduleName, ht.getModuleCategory(moduleName), functionName)):
        
        template = '{temp}\n\t\t\t# Execute, get result and show it\n'.format(temp=template)
        template = '{temp}\t\t\t{r}\n'.format(temp=template, r="result = ht.getModule('{moduleName}').{functionName}({functionParamsForCallInStr}{space_if_params})".format(moduleName=moduleName, functionName=functionName, functionParamsForCallInStr=functionParamsForCallInStr, space_if_params=' ' if functionParamsForCallInStr else ''))
        
        # If async
        template = '{temp}\t\t\tif request.POST.get(\'is_async_{f}\', False):\n'.format(temp=template, f=functionName)
        template = '{temp}\t\t\t\treturn JsonResponse({res})\n'.format(temp=template, res='{ "data" : returnAsModal(result) }')
                    
        example_return = "return renderMainPanel(request=request, popup_text=result)"
        template = '{temp}\t\t\t{r}\n'.format(temp=template, r=example_return)

    else:
        template = '{temp}\n\t\t\t# Execute the function\n'.format(temp=template)
        template = '{temp}\t\t\t{r}\n'.format(temp=template, r="ht.getModule('{moduleName}').{functionName}({functionParamsForCallInStr}{space_if_params})".format(moduleName=moduleName, functionName=functionName, functionParamsForCallInStr=functionParamsForCallInStr, space_if_params=' ' if functionParamsForCallInStr else ''))
        
        if not params or ('params' in params and not params['params'] and ('defaults' in params and not params['defaults'])):
            template = '{temp}\t\t\tpass\n'.format(temp=template)

		
			
    temp = 'return JsonResponse({ "data" : str(e) })\n\t\treturn renderMainPanel(request=request, popup_text=str(e))\n\t'
    template = '{temp}\texcept Exception as e:\n\t\tif request.POST.get(\'is_async_{f}\', False):\n\t\t\t{t}'.format(temp=template, f=functionName, t=temp)
    return template

def  __createViewFileForModule__(moduleName, category):
    categoryDir =  __getModuleViewCategoryDir__(moduleName, category)
    fileView =  __getModuleViewFilePath__(moduleName, category)
    if not os.path.isdir(categoryDir):
        os.mkdir(categoryDir)
    if not os.path.isfile(fileView):
        with open(fileView, 'w') as f:
            f.write(__default_view_init__)



from hackingtools.core import Logger, Utils, Config
if Utils.amIdjango(__name__):
	from .library.core import hackingtools as ht
else:
	import hackingtools as ht
import os

from pyshorteners import Shortener

config = Config.getConfig(parentKey='modules', key='ht_urlshortener')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():

	def __init__(self):
		self._main_gui_func_ = 'createShortcut'
		self._funcArgFromFunc_ = {
			'createShortcut' : {
				'domainShortener' : {
					'urlshortener' : 'getShortDomains' 
				}
			}
		}

	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_urlshortener'), debug_module=True)

	def createShortcut(self, url, fakeUrl, fakeText, domainShortener='tinyurl', api_key=None, user_id=None):
		try:
			short = Shortener(api_key=api_key, user_id=user_id)
			if domainShortener in short.available_shorteners:
				sh = eval( 'short.{do}'.format(do=domainShortener) )
				endLink = sh.short(url)
				withouthttp = endLink[7:]
				if withouthttp.startswith('/'):
					withouthttp = withouthttp[1:]
				fakeUrl = fakeUrl.replace('http://', '').replace('https://', '').replace('www.', '')
				fakeText = '-'.join( fakeText.split(' ') )
				return "https://www.{dom}-{post}@{withouthttp}".format(dom=fakeUrl, post=fakeText, withouthttp=withouthttp)
			return ""
		except Exception as e:
			Logger.printMessage(str(e), is_warn=True)
			return ""

	def getShortDomains(self):
		return Shortener().available_shorteners


from hackingtools.core import Logger, Utils, Config
if Utils.amIdjango(__name__):
	from hackingtools.core import hackingtools as ht
else:
	import hackingtools as ht
	
import os

config = Config.getConfig(parentKey='modules', key='ht_hash')
output_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'output'))

class StartModule():

	def __init__(self):
		self._main_gui_func_ = 'hashContent'

	def help(self):
		Logger.printMessage(message=ht.getFunctionsNamesFromModule('ht_hash'), debug_module=True)

	def hashContent(self, content):
		try:
			content.lower()
			contador = 1
			a = 23
			b = 35
			c = 36
			d = 75
			e = 72
			f = 24
			g = 63
			h = 62
			i = 56
			j = 63
			k = 19
			l = 37
			n = 47
			m = 48
			o = 634
			p = 745
			q = 85
			r = 756
			s = 4
			t = 48
			u = 36
			v = 473
			w = 7
			x = 7342
			y = 65
			z = 46
			pt = 342
			kh = 576
			valores = ("")
			if "a" in  content:
				
				contador = content.count("a")     
				ac = contador*a*b*contador*c*d*e*f**contador*g*h*i*j*k*l*m*n*s**t*u*v*contador*w*x*y*z*pt*contador
				
			if "b" in  content:
				contador = content.count("b")
				bc = a*pt*j*d*contador**l*f*t*contador**h*u*j*h*x*l*contador*l*o*z*u*v*c*c*q*z*pt
			if "c" in content:
				contador = content.count("c")
				cc = f*h*j*e*+f*f*g*contador*h*u*j*h*k*e*contador*a*o*z*u*v*c*c*q*h**s+h+contador**u*j*h*x*l*l
			if "d" in content:
				contador = content.count("d")
				dc = f*h*j**g*contador*h*e*+f*u*j*h*k*e*contador*a*e*+f*f*h**s+h+contador**u*j*h*+f*f*e+f*f*x*l*l
			if "e" in content:
				contador = content.count("e")
				ec = f*h*k*j*h*k*e*contador*a+h+contador**u*j*h*+f*f*e+f*f*e*+f*f*h**s*x*l*l
			if "f" in content:
				contador = content.count("f")
				fc = f*j*e+f+k*l*m*n*s**f*x+h*s+k*l*m*n*contador*s**d+f*a*l*l*contador
			if "g" in content:
				contador = content.count("g")
				gc = f*h*j*j*h*+f*f*e+f*f*x*l**g*contador*h*e**s+h+contador**u
			if "h" in content:
				contador = content.count("h")
				hc = f*h*j**g*contador*h*e*x+h*s+k*l*e*contador*h+contador**u*x+h*s+k*l*m
			if "i" in content:
				contador = content.count("i")
				ic = f*u*j*h*k*e*contador*a*e*+f*f+k*e*contador*a*e*+f*u*j*h*k*e*contador*a*e*+f*f*f*h**s+h+contador**u*j*h*+f*f*e+f*f*x*l*l
			if "j" in content:
				contador = content.count("j")
				jc = f*k*l*m*n*s**t*u*contador*j*h*k*e+j*h*k*e
			if "k" in content:
				contador = content.count("k")
				kc = f*h*f*f*e*+f*u*j*h*e+f*f*x*l*l*s+h+contador**u*j*h*+f*f*e+f*f*x*l*l
			if "l" in content:
				contador = content.count("l")
				lc = f*h*j*h*+f*f*e+f*f*x*l+h+contador**u*j*h*+f*f*e+f*f*x*l*l
			if "n" in content:
				contador = content.count("n")
				nc = f*h*j**g*contador*h*e*+j*h*e+f*f*x*l+f*x*l*l
			if "m" in content:
				contador = content.count("m")
				mc = f*h*j**g*contador*h*e*+f*u+u*j*h*+f*f*e+f*f*x*l*l
			if "o" in content:
				contador = content.count("o")
				oc = f*h*j**g*contador*h*e*+f*u*j*h*+f*f*e+f*f*x*l**x*l*l
			if "p" in content:
				contador = content.count("p")
				pc = f*h*j**g*contador*h*e*+f*h*+f*f*e+f*f*x*l**k*e+j*h*k*e
			if "q" in content:
				contador = content.count("q")
				qc = f*h*j**g*contador*h*e*+f*u*j*h*k*e+j*h*k*e+f*f*x*l*l
			if "r" in content:
				contador = content.count("r")
				rc = f*h*j**g*contador*h*e*+f*u*j*h**s+h+contador**u*j*h*+f*f*e+f*f*x*l*l
			if "s" in content:
				contador = content.count("s")
				sc = f*h*j**g*contador*h*e*+f*u*j*h*h*i*j*k*l*m*n*s**j*h*+h*i*j*k*l*m*n*s**f*e+f*f*x*l*l
			if "t" in content:
				contador = content.count("t")
				tc = f*h*h*i*j*k*l*m*n*s**t*h*k*e*contador*a*e+f*e+f*f*x*l*l
			if "u" in content:
				contador = content.count("u")
				uc = contador*h*e*+f*u*h*i*j*k*l*m*n*s**t*+f*f*e+f*f*x*l*l
			if "v" in content:
				contador = content.count("v")
				vc = f*h*j**g*contador*h*e*+f*u**l*m*u*j*h*k*e*n*s**t*k*h*+f*f*e+f*f*x*l*l
			if "w" in content:
				contador = content.count("w")
				wc = f*h*j**g*contador*h*e*+f*u*j*h*k*e*u*j*h*k*e*e+f*f*e+f*f*x**x*l*l
			if "x" in content:
				contador = content.count("x")
				xc = f*h*j**g*contador*h*e*+f*u*j*h*k*e*contador*a*e*+f*f*h**s+h+contador**u*j*h*+f*f*e+f*f*x*l*l
			if "y" in content:
				contador = content.count("y")
				yc = f*h*j*l*m*n*s**t*k+contador*u*j*h*+f*f*h**s+h+contador
			if "z" in content:
				contador = content.count("z")
				zc = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if " " in content:
				contador = content.count(" ")
				espacio = f*h*k*l*m*n*s*j+g*contador*h*e*i*j*k*j*k*l*m*n*s
			if "." in content:
				contador = content.count(".")
				punto = f*h**k*l+h*k*l*m*n*s*j+m*n*s*k*l*m*n*s+h-f**+i*j*k*l*m*n*s
			if "-" in content:
				contador = content.count("-")
				barra = +j+g*contador*h*s+i*j+k*l*m*n*s*k*l*m*n*s
			if "_" in content:
				contador = content.count("_")
				barrabaja = f*h**k*l*m*n*s*e*contador*a*h**s+i*j*k*l*m*n*s
			if "}" in content:
				contador = content.count("}")
				corcheteuno = f*h*j+g*contador*h*e*i*j*kh**s+i*j*k*l*m*n*s+l*m*n*s
			if "{" in content:
				contador = content.count("{")
				corchetedos = f*h*j+g*contador*h*e*i*j*k*l*l*m*n*s+l*m*n**contador*a*h**s+i*j*k*l*m*n*s
			if "+" in content:
				contador = content.count("+")
				mas = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if "*" in content:
				contador = content.count("*")
				asterisco = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if "," in content:
				contador = content.count(",")
				coma = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if ";" in content:
				contador = content.count(";")
				puntoycoma = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if ":" in content:
				contador = content.count(":")
				dospuntos = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if "1" in content:
				contador = content.count("1")
				uno = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if "2" in content:
				contador = content.count("2")
				dos = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if "3" in content:
				contador = content.count("3")
				tres = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if "4" in content:
				contador = content.count("4")
				cuatro = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if "5" in content:
				contador = content.count("5")
				cinco = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if "6" in content:
				contador = content.count("6")
				seis = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if "7" in content:
				contador = content.count("7")
				siete = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if "8" in content:
				contador = content.count("8")
				ocho = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if "9" in content:
				contador = content.count("9")
				nueve = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if "0" in content:
				contador = content.count("z")
				cero = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if "[" in content:
				contador = content.count("[")
				corchetetres = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if "]" in content:
				contador = content.count("]")
				corchetecuatro = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if '"' in content:
				contador = content.count('"')
				comilla = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if "'" in content:
				contador = content.count("'")
				comi = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if "=" in content:
				contador = content.count("z")
				igual = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if "(" in content:
				contador = content.count("(")
				parentesisa = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if ")" in content:
				contador = content.count(")")
				parentesisb = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if "!" in content:
				contador = content.count("!")
				exclamacion = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if "@" in content:
				contador = content.count("@")
				arroba = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if "#" in content:
				contador = content.count("#")
				almoadilla = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if "$" in content:
				contador = content.count("$")
				dinero = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if "/" in content:
				contador = content.count("/")
				barra = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if "´" in content:
				contador = content.count("´")
				tildea = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if "`" in content:
				contador = content.count("`")
				tildeb = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s
			if "¨" in content:
				contador = content.count("¨")
				tildec = f*h*j+g*contador*h*e*i*j*k*l*m*n*s**t*k*e*contador*a*h**s+i*j*k*l*m*n*s


			if "a" not in  content:
				ac = 45336
			if "b" not in  content:
				bc = 8363
			if "c" not in content:
				cc = 53524
			if "d" not in content:
				dc = 485353
			if "e" not in content:
				ec = 976524
			if "f" not in content:
				fc = 73562
			if "g" not in content:
				gc = 36455
			if "h" not in content:
				hc = 64572
			if "i" not in content:
				ic = 74522
			if "j" not in content:
				jc = 3635
			if "k" not in content:
				kc = 55534
			if "l" not in content:
				lc = 9846
			if "n" not in content:
				nc = 83654
			if "m" not in content:
				mc = 34133
			if "o" not in content:
				oc = 3735
			if "p" not in content:
				pc = 30372
			if "q" not in content:
				qc = 64522
			if "r" not in content:
				rc = 3524
			if "s" not in content:
				sc = 83624
			if "t" not in content:
				tc = 52233
			if "u" not in content:
				uc = 677563
			if "v" not in content:
				vc = 74386442
			if "w" not in content:
				wc = 2735222
			if "x" not in content:
				xc = 77454
			if "y" not in content:
				yc = 5343332
			if "z" not in content:
				zc = 84624
			if "." not in content:
				punto = 86454
			if "-" not in content:
				barra = 74536
			if " " not in content:
				espacio = 73253
			if "'" not in content:
				comi = 73535
			if "_" not in content:
				barrabaja = 73546
			if "{" not in content:
				corcheteuno = 764535
			if "}" not in content:
				corchetedos = 735334
			if "[" not in content:
				corchetetres = 734232
			if "]" not in content:
				corchetecuatro = 6346
			if "1" not in content:
				uno = 95635
			if "2" not in content:
				dos = 35232
			if "3" not in content:
				tres = 342527
			if "4" not in content:
				cuatro = 45242
			if "5" not in content:
				cinco = 84537
			if "6" not in content:
				seis = 36241
			if "7" not in content:
				siete = 24361
			if "8" not in content:
				ocho = 84523
			if "9" not in content:
				nueve = 51423
			if "0" not in content:
				cero = 34261
			if ";" not in content:
				puntoycoma = 94524
			if ":" not in content:
				dospuntos = 83521
			if "=" not in content:
				igual = 4535
			if "!" not in content:
				exclamacion = 72525
			if "@" not in content:
				arroba = 12322
			if "$" not in content:
				dinero = 74533
			if "/" not in content:
				barra = 3523
			if "´" not in content:
				tildea = 84724
			if "`" not in content:
				tildeb = 324
			if "¨" not in content:
				tildec = 64835
			if "+" not in content:
				mas = 84624
			if ")" not in content:
				parentesisa = 84624
			if "(" not in content:
				parentesisb = 84624

			return str(ac*cc*bc+dc*ec+fc*gc*hc*ic+jc+kc+lc*ac*nc*mc*oc-pc*qc+rc+sc-tc*uc*wc*xc*yc*zc-rc+sc-tc*nc*a-barra*tildea+tildeb+tildec*mas+parentesisa*parentesisb+dinero+dinero*exclamacion+igual+puntoycoma-dospuntos+uno+dos*3+cuatro*cinco+seis*ocho+nueve*cero+corcheteuno-corchetedos*corchetetres+corchetecuatro+barrabaja+comi+espacio+barra-punto)
		except Exception as e:
			return str(e)
